/*
 *  The graphical definition element of deltaFSD
 *  deletion.c - handles deletion of functions and various graphical elements  
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */    
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */

# include <stdlib.h>
# include "engine.h"
# include "gutils.h"
# include "deletion.h"

// The deletion components
//
void delete_segments (lsp sp)
{ /* we need to work back along the chain */
	if (sp->next) delete_segments (sp->next);
	free (sp);
}
void delete_from_line_list (linp *lp, linp dlp)
{ /* work down the list of lines - remove any matching the line given */
/* this version doesn't cope with an empty list - ie no initial address */
	if (*lp) delete_from_line_list (&((*lp)->next), dlp); /* until we've reached the end of the list */ 
	if (*lp == dlp) /* we've got a match */
		(*lp) = (*lp)->next; /* stitch it out of the list */
}
void delete_function_dep (funp *fpp, funs fs)
{
funp fp;
	fp = *fpp;
	if (fp) delete_function_dep (&(fp->next), fs);
	if (fp && fp->depfun == fs) /* we've got a match */
		{
		(*fpp) = fp->next; /* stitch it out of the list */
		free (fp);
		}
}
void delete_line (linp lp)
{	/* first stop the input variable for the destinatation stub pointing at the source */
dstubp dp;
sstubp sp;
	dp = lp->dststub ? get_final_destination(lp->dststub) : NULL; /* finds the functional end of line */
	sp = lp->srcstub ? get_original_source(lp->srcstub) : NULL;
	if (dp) dp->vp->ovarb = NULL; /* we need to break the connection to this function */ 
		/* then disconnect the line - if either end is connected to a stub */
	delete_from_line_list (&(lp->srcstub->firstl), lp);  /* one from many line from the source stub */
	if (lp->dststub) lp->dststub->lp = NULL; /* only one to destination */
	if (sp && sp->vp->frstf && lp->dststub) /* the stub exists and its variable has at least one dependant function */ 
		delete_function_dep (&(sp->vp->frstf), lp->dststub->bp->fp);
	delete_segments (lp->frstseg); /* remove the segments */
	/* and now the line itself */
	free (lp);	
}
void delete_variable (varo vp)
{
funp fp;
dstubp sp;
	for (fp = vp->frstf; fp != NULL; fp = fp->next) /* look through each of the dependent functions */
		for (sp = fp->depfun->bp->dsbp; sp != NULL; sp = sp->next)
			if (sp->vp->ovarb == vp) sp->vp->ovarb = NULL; /* clear the input reference */
	free (vp); /* release the data structure */			 
}
void delete_output_variables (funs fp)
{
	if (fp->frstout) free (fp->frstout);
	fp->frstout = NULL;
}
void delete_from_func_list (funp *fpp, vari vp)
{
funp fp;
	fp = *fpp;	
	if (fp) delete_from_func_list (&(fp->next), vp); /* we've reached the end of the list */ 
	if (fp && fp->depvar == vp) /* we've got a match */
		{ 
		(*fpp) = fp->next; /* stitch it out of the list */
		free (fp);
		}
}
                                                                  
void delete_input_variables (funs fp)
{ 
int counter;
	/* remove this function from the list of those dependant on each input variable */
	for (counter = 0; counter < fp->ins; counter++)
		if ((fp->frstin[counter]).ovarb)
			delete_from_func_list (&((fp->frstin[counter]).ovarb->frstf), &(fp->frstin[counter]));
	if (fp->frstin) free (fp->frstin);
	fp->frstin = NULL;
}
void delete_lines (linp lp)
{
	if (lp->next) delete_lines (lp->next);
	delete_line (lp);
}
void delete_sstubs_and_lines (sstubp ssbp)
{
	if (ssbp->next) delete_sstubs_and_lines (ssbp->next);
	if (ssbp->firstl) delete_lines (ssbp->firstl);
//	if (ssbp->lp) delete_line (ssbp->lp);	/* delete the lines that exist */
	free (ssbp); /* and the stub itself */ 
}
void delete_dstubs_and_lines (dstubp dsbp)
{ 
	if (dsbp->next) delete_dstubs_and_lines (dsbp->next);
	if (dsbp->lp) delete_line (dsbp->lp);	/* delete the lines that exist */
	free (dsbp); /* and the stub itself */ 
}
void delete_function (funs fp)
{
	/* first remove any output variables */
	delete_output_variables (fp); 
	/* then all the input variables */
	delete_input_variables (fp); 
	/* remove the function */
	free (fp);
}
void delete_box (boxp bp)
{
	/* disconect the box by removing the structures accociated with the box's stubs */
	if (bp->ssbp) delete_sstubs_and_lines (bp->ssbp);
	bp->ssbp = NULL; /* make sure we dont point at it */
	if (bp->dsbp) delete_dstubs_and_lines (bp->dsbp);
	bp->dsbp = NULL; /* make sure we dont point at it */
	/* modules will have the (common) function deleted as part of the parent - every other box has a function */
	if (!bp->prnt) delete_function (bp->fp); /* then delete the associated function */
	/* stitch the box out of the chain */
	if (bp->next) bp->next->prev = bp->prev;	/* the last in the chain wont have a next */
	else if (bp->prev) bp->prev->next = NULL;
	if (bp->prev) bp->prev->next = bp->next;
	else if (bp->next) bp->next->prev = NULL;
	free (bp); /* and finally the box itself */
}

