/*
 *  The graphical definition element of deltaFSD
 *  engine.h - header for engine.c - function instantiation and the core exeution engine 
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */    
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */



#ifndef _ENGINE_H_
#define _ENGINE_H_

# include "fileops.h"
//
// We have
// Variables and pointers to variables
// Functions and pointers to functions
//
typedef struct Vin *vari;
typedef struct Vout *varo;
typedef struct Fp *funp;	// points to an element in the list of dependant functions
typedef struct Fs *funs;	// points to a function instance
typedef struct Ft *futs;	// points to a function

typedef struct box* boxp;
typedef struct line* linp;

typedef void (*funcp)(funs f);
typedef void (*functp)(futs f);
typedef void (*funcbp)(boxp bp);
typedef void (*funcw)(xmlNodePtr node, boxp bp);
typedef void (*funcc)(xmlNodePtr node, funs fp);

void execute (varo curo);
void process_func (funs curf);
void process_var (varo curo);
//
struct Vin
	{ // an element in the array of input variables for a function
	varo ovarb;
// fix	float /* for now */ deflt; /* modify the defaults in the function.h to pick these up - ie not 0.0 */
	};
//
struct Vout
	{ // an element in the array of output variables for a function
	float /* for now */ current_value;
	float next_value;
	funp frstf; /* the head of the list of functions that depend on this variable */
	};
//
struct Fp
	{ // an element in the list of dependant functions
	funs depfun;
	vari depvar;
	funp next;
	};
//
//  Each function instance has parameters
//	name
//	the function pointer
//	some instance data 
//	a pointer to the first input variable
//	a pointer to the first output variable
//	the number of inputs
//	the number of outputs
//
struct Fs
	{ // an element in the list of function instances
	futs funct;   // its parent function
	vari frstin;  // head of list of inputs
	varo frstout; // head of list of outputs
	int counts;     // counts through outputs 
	int ins;      // the number of inputs	
	int outs;     // the number of outputs
	boxp bp;	      // the box it draws 
	};
//
// some functions need (deserve) special treatment
// ie special functions
// most (?) functions can use the standard handlers
// seperating these off keeps the standardfunctions simple
struct Fx
	{
	functp fregstr;	// when the type of function is created
	funcp finit;	// create instance from UI
	funcbp frqst;	// called on right click
	funcc fcreate;	// create instance from file
	funcw fwrite;	// genrate text version of configuration
	funcp fdestroy; // any per instance clean up 
	char *describe;
	};
	
typedef struct Fx *fexts;	// points to the set of special functions

struct Ft
	{ // an element in the list of functions
	char *mentry; // the menu entry (with /group/)
	char *fname;  // the name of the function 
	funcp func;   // the function itself
	fexts funcext; // the structure of special functions (if any)
	int ins;      // the number of inputs
	int outs;     // the number of outputs
	};
	
void process_vars (funs f);	
void process_func (funs f);
void process_var (varo curo);
funs function_insert (futs funct, boxp bp);
void add_funcio (funs fs);
void link_var (linp lp);
void complete_line (linp lp);

#endif /* _ENGINE_H_ */

