
/*
 *	The graphical definition element of deltaFSD
 *	fchart.c - the various components of the chart function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_chart (futs f);
void instance_chart (funs fp);
void popup_chart (boxp bp);
void create_chart (xmlNodePtr node, funs fp);
void write_chart (xmlNodePtr node, boxp bp);
void remove_chart (funs fp);


struct Fx chart_functions = {
	init_chart,			/* when the chart function type is created */
	instance_chart,		/* create instance from UI */
	popup_chart,		/* right click on hotspot */
	create_chart,		/* create instance from file */
	write_chart,		/* generate text version of configuration */
	remove_chart,		/* any per instance clean up */
	"The chart will have -1 < y < +1\nSet scale and offset to produce values in this range\nThe second input advances the chart\n(on zero -> non-zero transition"
	};

/* Graphing output macros */
# define CHART_XSIZE 100.0
# define CHART_YSIZE 100.0
# define CHART_XMARGIN 5.0
# define CHART_YMARGIN 5.0
# define CHART_XMIN (CHART_XMARGIN)
# define CHART_YMIN (CHART_YMARGIN)
# define CHART_XCENTER (1.0*CHART_XSIZE + CHART_XMARGIN)
# define CHART_YCENTER (1.0*CHART_YSIZE + CHART_YMARGIN)
# define CHART_XMAX (2.0*CHART_XSIZE + CHART_XMARGIN)
# define CHART_YMAX (2.0*CHART_YSIZE + CHART_YMARGIN)
# define CHART_WINW (2.0*CHART_XSIZE + 2.0*CHART_XMARGIN)
# define CHART_WINH (2.0*CHART_YSIZE + 2.0*CHART_YMARGIN)
# define VAL_ARRAY ((int)(2*CHART_XSIZE))

# define WINDOW_HEIGHT (GTK_WIDGET(window)->allocation.height)
# define WINDOW_WIDTH (GTK_WIDGET(window)->allocation.width)

/* now those using the data structure */
# define CHART_PREVX (IDP->prev_x)
# define CHART_IN (In(0))
# define CHART_YSCALE (IDP->scale)
# define CHART_YSCALED (CHART_IN * CHART_YSCALE * CHART_YSIZE)
# define CHART_YAXISPOS (CHART_YCENTER - ((IDP->offset) * CHART_YSIZE))
# define CHART_PIXBIT (CHART_YAXISPOS - CHART_YSCALED)

typedef float values[VAL_ARRAY];


typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*scale_entry;
	GtkWidget	*offset_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	float	prev_x;
	GdkPixmap *	pixmap;
	float	offset;
	float	scale;
	GtkWindow *	window;
	values	prevs;
	} instance_data;
	
void redraw_chart (GtkWindow *window);
void clear_chart (GtkWidget* w, boxp bp);
void create_chart_window (instance_data* idp);


/* Chart function */
void Chart (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{
int i;
	if (!(IDP->window))  return; /* the window has gone away */
	if (!(fp->frstin->ovarb)) return; /* no input variable set up yet */
	i = !In(1) || IDP->prev_x;
	IDP->prev_x = In(1); /* update the reference */
	if (i) return; /* only update if the x axis value has transitioned from zero */
	/* put in the x axis */
	gdk_draw_line (IDP->pixmap, root_window.gray, CHART_XMIN, CHART_YAXISPOS, CHART_XMAX, CHART_YAXISPOS);
	for (i = 0; i < VAL_ARRAY; i++)
		gdk_draw_rectangle (IDP->pixmap, root_window.white, FALSE, i+CHART_XMARGIN, IDP->prevs[i], 1, 1);
	for (i = 0; i < VAL_ARRAY-1; i++)
		IDP->prevs [i] = IDP->prevs[i+1];
	if ((CHART_PIXBIT >= CHART_YMIN) && (CHART_PIXBIT <= CHART_YMAX))
		IDP->prevs[VAL_ARRAY-1] = CHART_PIXBIT;
	else	IDP->prevs[VAL_ARRAY-1] = -1; /* the "do not draw" value */
	for (i = 0; i < VAL_ARRAY; i++)
		if (IDP->prevs[i] > -1)
			gdk_draw_rectangle (IDP->pixmap, root_window.black, FALSE, i+CHART_XMARGIN, IDP->prevs[i], 1, 1);
	redraw_chart(IDP->window);
}


void redraw_chart (GtkWindow *window)
{ /* update all of window area */
GdkRectangle update_rect;
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = GTK_WIDGET(window)->allocation.width;
	update_rect.height = GTK_WIDGET(window)->allocation.height;
	gtk_widget_draw (GTK_WIDGET(window), &update_rect);
}
static gint expose_chart (GtkWidget *widget, GdkEventExpose *event, instance_data* idp)
{
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], idp->pixmap,
	event->area.x, event->area.y, event->area.x, event->area.y,
	event->area.width, event->area.height);
	return FALSE;
}
void blank_chart (instance_data* idp)
# define IDP (idp)
{
GtkWindow* window = idp->window;
int i;
	gdk_draw_rectangle (idp->pixmap, root_window.white, TRUE, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
	/* draw the axis */
	gdk_draw_line (idp->pixmap, root_window.gray, CHART_XMIN, CHART_YAXISPOS, CHART_XMAX, CHART_YAXISPOS);
	redraw_chart(idp->window);
	/* clear the history array */
	for (i = 0; i < VAL_ARRAY; i++) idp->prevs[i] = -1; /* the "do not draw" value */
}
void clear_chart (GtkWidget* w, boxp bp)
{
	blank_chart ((instance_data*)bp->idata);
}

static gint configure_chart (GtkWidget *widget, GdkEventConfigure *event, instance_data* idp)
{
	if (idp->pixmap) gdk_pixmap_unref(idp->pixmap);
	idp->pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	blank_chart (idp);
	return TRUE;
}

void chart_quit(GtkWidget *widget, instance_data* idp)
{ /* the chart window is disappearing - flag no more charting possible */
	idp->window = NULL;
}

void create_chart_window (instance_data* idp)
{
GtkWidget *drawing_area, *vbox;
	idp->window = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
	vbox = gtk_vbox_new (FALSE, 0); /* create a vbox */
	gtk_container_add (GTK_CONTAINER (idp->window), vbox);
	drawing_area = gtk_drawing_area_new (); /* and a drawing area */
	gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), CHART_WINW,CHART_WINH);
	gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event", (GtkSignalFunc) expose_chart, idp);
	gtk_signal_connect (GTK_OBJECT(drawing_area), "configure_event", (GtkSignalFunc) configure_chart, idp);
	gtk_signal_connect (GTK_OBJECT (idp->window), "destroy", GTK_SIGNAL_FUNC (chart_quit), idp);
	gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK);
	gtk_window_set_title ((GtkWindow *)idp->window, idp->text2);
	gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
	gtk_widget_show (vbox);
	gtk_widget_show (drawing_area);
	gtk_widget_show (GTK_WIDGET(idp->window));
}




void make_chart (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the chart start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	create_chart_window (IDP);
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void chart_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->scale_entry));
	sscanf(entry_text, "%f", &(IDP->scale));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->offset_entry));
	sscanf(entry_text, "%f", &(IDP->offset));
	gtk_widget_destroy (IDP->dwindow);
	make_chart (bp);
}

void init_chart (futs f)
{
}

void setup_chart_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (chart_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->scale_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Scale", "1.0");
	IDP->offset_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Offset", "0.0");
}
void instance_chart (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_chart_dialog (fp->bp);
}

void remove_chart (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
	if (IDP->window) gtk_widget_destroy (GTK_WIDGET(IDP->window)); /* remove the old graph window */
	IDP->window = NULL; /* flag no more graphing possible */
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Chart destroyed");
}

void popup_delete_chart (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_chart (bp->fp);
}
void popup_adjust_chart (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
	if (IDP->window) gtk_widget_destroy (GTK_WIDGET(IDP->window)); /* remove the old graph window */
	IDP->window = NULL; /* flag no more graphing possible */
}

	setup_chart_dialog (bp);
}

void popup_chart (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Chart",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_chart,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_chart,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },
	{ "/Clear chart",	   NULL,	   clear_chart,		0,	  "<Item>" }

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;

	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_chart (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->scale = get_prop_val (node, "scale");
	IDP->offset = get_prop_val (node, "offset");
	make_chart (fp->bp);
}
void write_chart (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "scale", IDP->scale);
	set_prop_val (node, "offset", IDP->offset);
}
