/*
 *  The graphical definition element of deltaFSD
 *  ffilter.c - the various components of the filter function 
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */	
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */



# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "main.h" /* for the vbox (at bottom of main window) */
# include "function.h"
# include "deletion.h"

# define taps 35
// 34th order Kaiser filter
double lp[] = { -2.127012e-4, -1.434315e-3, -1.523899e-3, 1.523594e-3,
5.469842e-3, 3.952406e-3, -5.506853e-3, -1.396630e-2, -7.244833e-3,
1.517910e-2, 3.026919e-2, 1.068386e-2, -3.844119e-2, -6.620962e-2,
-1.330611e-2, 1.260324e-1, 2.799168e-1, 3.470214e-1, 2.799168e-1,
1.260324e-1, -1.330611e-2, -6.620962e-2, -3.844119e-2, 1.068386e-2,
3.026919e-2, 1.517910e-2, -7.244833e-3, -1.396630e-2, -5.506853e-3,
3.952406e-3, 5.469842e-3, 1.523594e-3, -1.523899e-3, -1.434315e-3,
-2.127012e-4};

double bp[] = { -6.238957e-3, 0.0, 1.389077e-3, -7.818848e-3,
-2.251760e-2, -2.874291e-2, -1.432263e-2, 1.935535e-2, 5.417910e-2,
6.508222e-2, 3.747954e-2, -1.967865e-2, -7.569528e-2, -9.558527e-2,
-6.276540e-2, 8.218622e-3, 7.842754e-2, 1.074919e-1, 7.842754e-2,
8.218622e-3, -6.276540e-2, -9.558527e-2, -7.569528e-2, -1.967865e-2,
3.747954e-2, 6.508222e-2, 5.417910e-2, 1.935535e-2, -1.432263e-2,
-2.874291e-2, -2.251760e-2, -7.818848e-3, 1.389077e-3, 0.0,
-6.238957e-3};

double hp[] = {-3.671040e-4, -1.433603e-3, -2.698704e-4, 3.431724e-3,
3.629797e-3, -3.950445e-3, -1.020383e-2, -1.033758e-3, 1.707373e-2,
1.517157e-2, -1.685950e-2, -3.861338e-2, -1.937564e-3, 6.617677e-2,
5.988269e-2, -8.870710e-2, -3.004827e-1, 5.962756e-1, -3.004827e-1,
-8.870710e-2, 5.988269e-2, 6.617677e-2, -1.937564e-3, -3.861338e-2,
-1.685950e-2, 1.517157e-2, 1.707373e-2, -1.033758e-3, -1.020383e-2,
-3.950445e-3, 3.629797e-3, 3.431724e-3, -2.698704e-4, -1.433603e-3,
-3.671040e-4};


struct Fx filter_functions =
{
	NULL,		// when the type of function is created
	instance_filter,	// create instance from UI
	NULL,	// called on right click
	create_filter,	// create instance from file
	write_filter,	// genrate text version of configuration
	NULL,		 // any per instance clean up 
	"Creates a simple digital filter.\n"	// the descriptive text
	"The sample rate (input 2) determines the frequency response.\n"
	"For the low and high pass the frequency = 1/8 of the sample rate.\n"
	"The band pass allows frequencies between 1/2 and 1/4\n"
	"of the sample rate."
};

typedef struct {
	GtkWidget *name_entry;
	GtkWidget *lp_button;
	GtkWidget *bp_button;
	GtkWidget *hp_button;
	double *coeff; /* lp, hp, bp */
	double values[taps];
	double prev_s; /* only update on sample */
	GtkWidget *dwindow; /* the setup dialog box window */
	funs fp;
} filter_data;

/* Filter input */
void Filter (funs fp)
{
filter_data *fdp = (filter_data *)fp->bp->idata;
int i;
	 if (In(1) == fdp->prev_s) return;
	 fdp->prev_s = In(1);
	 fdp->values[0] = In(0);
	 Out(0) = 0.0; 
	 for (i=0; i<taps; i++) Out(0) += fdp->coeff[i]*fdp->values[i];
	 for (i=taps-1; i>0; i--) fdp->values[i] = fdp->values[i-1];
}

void make_filter (filter_data *fdp)
{
int i;
	for (i=0; i<taps; i++) fdp->values[i] = 0.0;
	if (!strcmp ("High pass", fdp->fp->bp->text3)) fdp->coeff = hp;
	if (!strcmp ("Low pass", fdp->fp->bp->text3)) fdp->coeff = lp;
	if (!strcmp ("Band pass", fdp->fp->bp->text3)) fdp->coeff = bp;
	update_and_redraw (fdp->fp); /* required ?? - fix */
}

void filter_button_press (filter_data* fdp)
{
gchar *entry_text;
// extract the name
	entry_text = gtk_entry_get_text (GTK_ENTRY(fdp->name_entry));
	fdp->fp->bp->text2 = malloc (BUFFER_SIZE);
	strncpy (fdp->fp->bp->text2, entry_text, BUFFER_SIZE);
//
	fdp->fp->bp->text3 = malloc (BUFFER_SIZE);
// 
	if (((GtkToggleButton *)(fdp->lp_button))->active)
		strncpy (fdp->fp->bp->text3, "Low pass", BUFFER_SIZE);
	if (((GtkToggleButton *)(fdp->hp_button))->active)
		strncpy (fdp->fp->bp->text3, "High pass", BUFFER_SIZE);
	if (((GtkToggleButton *)(fdp->bp_button))->active)
		strncpy (fdp->fp->bp->text3, "Band pass", BUFFER_SIZE);
	gtk_widget_destroy (fdp->dwindow); /* done with the setup window */
	make_filter (fdp);
}
void setup_filter_dialog (filter_data* fdp)
{ /* utility used on setup and adjust */
GSList *group;
GtkWidget *but_box;
	fdp->dwindow = create_entry_vbox (filter_button_press, (GtkObject*)fdp);/* cast ?? */
// add the caption
	text_info ((GTK_DIALOG (fdp->dwindow)->vbox), fdp->fp->funct->funcext->describe);
// the vbox of the dialogue contains two prompt entry pairs
	fdp->name_entry = prompt_entry ((GTK_DIALOG (fdp->dwindow)->vbox), "Filter name", "?");
//
	but_box = STD_HBOX;
	fdp->lp_button = gtk_radio_button_new_with_label (NULL, "Low pass");
	STD_PACK (GTK_BOX (but_box), fdp->lp_button);
//	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(fdp->dwindow)->vbox), fdp->lp_button, TRUE, TRUE, 0);
	gtk_widget_show (fdp->lp_button);

	group = gtk_radio_button_group (GTK_RADIO_BUTTON (fdp->lp_button));
	fdp->bp_button = gtk_radio_button_new_with_label (group, "Band pass");
	STD_PACK (GTK_BOX (but_box), fdp->bp_button);
	gtk_widget_show (fdp->bp_button);

	group = gtk_radio_button_group (GTK_RADIO_BUTTON (fdp->bp_button));
	fdp->hp_button = gtk_radio_button_new_with_label (group, "High pass");
	STD_PACK (GTK_BOX (but_box), fdp->hp_button);
	gtk_widget_show (fdp->hp_button);
	STD_PACK (GTK_BOX (GTK_DIALOG(fdp->dwindow)->vbox), but_box);
	gtk_widget_show (but_box);
}
void instance_filter (funs f)
{
filter_data *fdp;
	fdp = malloc (sizeof (*fdp));  /* create the data area */
	fdp->fp = f;
	f->bp->idata = fdp; 
	setup_filter_dialog (fdp);
}

void create_filter (xmlNodePtr node, funs f)
{
filter_data *fdp;
	fdp = malloc (sizeof (*fdp));  /* create the data area */
	fdp->fp = f;
	f->bp->idata = fdp;
//
	f->bp->text2 = malloc(BUFFER_SIZE); /* will hold the name */
	strncpy (f->bp->text2, xmlGetProp (node, "name"), BUFFER_SIZE);
	f->bp->text3 = malloc(BUFFER_SIZE); /* will hold the type */
	strncpy (f->bp->text3, xmlGetProp (node, "type"), BUFFER_SIZE);
//
	make_filter (fdp);
}

void write_filter (xmlNodePtr node, boxp bp)
{
	xmlSetProp(node, "name", bp->text2);
	xmlSetProp(node, "type", bp->text3);
}
