
/*
 *	The graphical definition element of deltaFSD
 *	fget.c - the various components of the get function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_get (futs f);
void instance_get (funs fp);
void popup_get (boxp bp);
void create_get (xmlNodePtr node, funs fp);
void write_get (xmlNodePtr node, boxp bp);
void remove_get (funs fp);


struct Fx get_functions = {
	init_get,			/* when the get function type is created */
	instance_get,		/* create instance from UI */
	popup_get,		/* right click on hotspot */
	create_get,		/* create instance from file */
	write_get,		/* generate text version of configuration */
	remove_get,		/* any per instance clean up */
	"Enter details for user entry"
	};



typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*value_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	std_buffer	values;
	path_buffer	prompt;
	} instance_data;
	
void get_callback (GtkWidget *widget, instance_data *idp);


/* Get function */
void Get (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{ /* not actually called - new value is loaded directly into Out0 by the callback */
}


void get_callback (GtkWidget *widget, instance_data *idp)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(idp->value_entry));
	sscanf (entry_text, "%f", &(idp->fp->frstout->next_value));
	snprintf (idp->text3, BUFFER_SIZE, "= %.4f", idp->fp->frstout->next_value);
	update_and_redraw (idp->fp);
}




void make_get (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the get start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	snprintf (IDP->prompt, PATH_BUFFER_SIZE, "Enter value for %s :", IDP->text2);
	snprintf (IDP->text3, BUFFER_SIZE, "%.4f", IDP->fp->frstout->next_value);
	/* create the entry box */
	IDP->value_entry = prompt_entry (root_window.main_vbox, IDP->prompt, IDP->text3);
	gtk_signal_connect(GTK_OBJECT(IDP->value_entry), "activate", get_callback, IDP);
	update_and_redraw (IDP->fp);
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void get_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->value_entry));
	sscanf(entry_text, "%f", &(IDP->fp->frstout->next_value));
	gtk_widget_destroy (IDP->dwindow);
	make_get (bp);
}

void init_get (futs f)
{
}

void setup_get_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (get_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->value_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Value", "1.0");
}
void instance_get (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_get_dialog (fp->bp);
}

void remove_get (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
	gtk_container_remove (GTK_CONTAINER(IDP->value_entry->parent->parent), IDP->value_entry->parent);
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Get destroyed");
}

void popup_delete_get (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_get (bp->fp);
}
void popup_adjust_get (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
	gtk_container_remove (GTK_CONTAINER(IDP->value_entry->parent->parent), IDP->value_entry->parent);
}

	setup_get_dialog (bp);
}

void popup_get (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Get",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_get,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_get,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_get (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->fp->frstout->next_value = get_prop_val (node, "value");
	make_get (fp->bp);
}
void write_get (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "value", IDP->fp->frstout->next_value);
}
