
/*
 *	The graphical definition element of deltaFSD
 *	fgraph.c - the various components of the graph function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_graph (futs f);
void instance_graph (funs fp);
void popup_graph (boxp bp);
void create_graph (xmlNodePtr node, funs fp);
void write_graph (xmlNodePtr node, boxp bp);
void remove_graph (funs fp);


struct Fx graph_functions = {
	init_graph,			/* when the graph function type is created */
	instance_graph,		/* create instance from UI */
	popup_graph,		/* right click on hotspot */
	create_graph,		/* create instance from file */
	write_graph,		/* generate text version of configuration */
	remove_graph,		/* any per instance clean up */
	"The graph will have -1 < x < +1, -1 < y < +1\nSet X and Y Scale, X and Y Offset\nto produce values in this range"
	};

/* Graphing output macros */
# define GRAPH_XSIZE 100.0
# define GRAPH_YSIZE 100.0
# define GRAPH_XMARGIN 5.0
# define GRAPH_YMARGIN 5.0
# define GRAPH_XMIN (GRAPH_XMARGIN)
# define GRAPH_YMIN (GRAPH_YMARGIN)
# define GRAPH_XCENTER (1.0*GRAPH_XSIZE + GRAPH_XMARGIN)
# define GRAPH_YCENTER (1.0*GRAPH_YSIZE + GRAPH_YMARGIN)
# define GRAPH_XMAX (2.0*GRAPH_XSIZE + GRAPH_XMARGIN)
# define GRAPH_YMAX (2.0*GRAPH_YSIZE + GRAPH_YMARGIN)
# define GRAPH_WINW (2.0*GRAPH_XSIZE + 2.0*GRAPH_XMARGIN)
# define GRAPH_WINH (2.0*GRAPH_YSIZE + 2.0*GRAPH_YMARGIN)
/* now those using the data structure */
# define GRAPH_PREVX (IDP->prev_x)
# define GRAPH_INX (In(1))
# define GRAPH_INY (In(0))
# define GRAPH_XSCALE (IDP->xscale)
# define GRAPH_YSCALE (IDP->yscale)
# define GRAPH_SCALEDX (GRAPH_INX * GRAPH_XSCALE * GRAPH_XSIZE)
# define GRAPH_SCALEDY (GRAPH_INY * GRAPH_YSCALE * GRAPH_YSIZE)
# define GRAPH_XAXISPOS (GRAPH_XCENTER + ((IDP->xoffset) * GRAPH_XSIZE))
# define GRAPH_YAXISPOS (GRAPH_YCENTER - ((IDP->yoffset) * GRAPH_YSIZE))
# define GRAPH_XPIXPOS (GRAPH_XAXISPOS + GRAPH_SCALEDX)
# define GRAPH_YPIXPOS (GRAPH_YAXISPOS - GRAPH_SCALEDY)

# define WINDOW_HEIGHT (GTK_WIDGET(IDP->window)->allocation.height)
# define WINDOW_WIDTH (GTK_WIDGET(IDP->window)->allocation.width)


typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*xscale_entry;
	GtkWidget	*xoffset_entry;
	GtkWidget	*yscale_entry;
	GtkWidget	*yoffset_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	float	prev_x;
	GdkPixmap *	pixmap;
	GtkWindow *	window;
	float	yscale;
	float	yoffset;
	float	xoffset;
	float	xscale;
	} instance_data;
	
void redraw_graph (GtkWindow *window);
void clear_graph (GtkWidget* w, boxp bp);
void create_graph_window (instance_data* idp);


/* Graph function */
void Graph (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{/* we avoid double dotting by only updating if the y axis value has been modified */
	if (!fp->frstin->ovarb)  return; /* no input variable set up yet */
	if (!IDP->window)  return; /* the window has gone away */
	if (GRAPH_INX == IDP->prev_x)  return; /* only update if the x axis value has been modified */
	IDP->prev_x = GRAPH_INX; /* update the reference */
	gdk_draw_rectangle (IDP->pixmap, root_window.black, FALSE, GRAPH_XPIXPOS, GRAPH_YPIXPOS, 1, 1);
	redraw_graph(IDP->window);
}


void redraw_graph (GtkWindow *window)
{ /* update all of window area */
GdkRectangle update_rect;
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = GTK_WIDGET(window)->allocation.width;
	update_rect.height = GTK_WIDGET(window)->allocation.height;
	gtk_widget_draw (GTK_WIDGET(window), &update_rect);
}

static gint expose_graph (GtkWidget *widget, GdkEventExpose *event, instance_data* idp)
{
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], idp->pixmap,
	event->area.x, event->area.y, event->area.x, event->area.y,
	event->area.width, event->area.height);
	return FALSE;
}

void blank_graph (instance_data* idp)
# define IDP (idp)
{
	gdk_draw_rectangle (IDP->pixmap, root_window.white, TRUE, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
	/* draw the axes */
	gdk_draw_line (IDP->pixmap, root_window.gray, GRAPH_XAXISPOS, GRAPH_YMIN, GRAPH_XAXISPOS, GRAPH_YMAX);
	gdk_draw_line (IDP->pixmap, root_window.gray, GRAPH_XMIN, GRAPH_YAXISPOS, GRAPH_XMAX, GRAPH_YAXISPOS);
	redraw_graph(IDP->window);
}

void clear_graph (GtkWidget* w, boxp bp)
{
	blank_graph ((instance_data*)bp->idata);
}

static gint configure_graph (GtkWidget *widget, GdkEventConfigure *event, instance_data* idp)
{
	if (idp->pixmap) gdk_pixmap_unref(idp->pixmap);
	idp->pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	blank_graph (idp);
	return TRUE;
}

void graph_quit(GtkWidget *widget, instance_data* idp)
{ /* the graph window is disappearing - flag no more graphing possible */
	idp->window = NULL;
}

void create_graph_window (instance_data* idp)
{
GtkWidget *drawing_area, *vbox;
	idp->window = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
	vbox = gtk_vbox_new (FALSE, 0); /* create a vbox */
	gtk_container_add (GTK_CONTAINER (idp->window), vbox);
	drawing_area = gtk_drawing_area_new (); /* and a drawing area */
	gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), GRAPH_WINW,GRAPH_WINH);
	gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event", (GtkSignalFunc) expose_graph, idp);
	gtk_signal_connect (GTK_OBJECT(drawing_area), "configure_event", (GtkSignalFunc) configure_graph, idp);
	gtk_signal_connect (GTK_OBJECT (idp->window), "destroy", GTK_SIGNAL_FUNC (graph_quit), idp);
	gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK);
	gtk_window_set_title (idp->window, idp->text2);
	gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
	gtk_widget_show (vbox);
	gtk_widget_show (drawing_area);
	gtk_widget_show (GTK_WIDGET(idp->window));
}




void make_graph (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the graph start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	create_graph_window (IDP);
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void graph_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->xscale_entry));
	sscanf(entry_text, "%f", &(IDP->xscale));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->xoffset_entry));
	sscanf(entry_text, "%f", &(IDP->xoffset));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->yscale_entry));
	sscanf(entry_text, "%f", &(IDP->yscale));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->yoffset_entry));
	sscanf(entry_text, "%f", &(IDP->yoffset));
	gtk_widget_destroy (IDP->dwindow);
	make_graph (bp);
}

void init_graph (futs f)
{
}

void setup_graph_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (graph_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->xscale_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "X Scale", "1.0");
	IDP->xoffset_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "X Offset", "0.0");
	IDP->yscale_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Y Scale", "1.0");
	IDP->yoffset_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Y Offset", "0.0");
}
void instance_graph (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_graph_dialog (fp->bp);
}

void remove_graph (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
	if (IDP->window) gtk_widget_destroy (GTK_WIDGET(IDP->window)); /* remove the old graph window */
	IDP->window = NULL; /* flag no more graphing possible */
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Graph destroyed");
}

void popup_delete_graph (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_graph (bp->fp);
}
void popup_adjust_graph (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
	if (IDP->window) gtk_widget_destroy (GTK_WIDGET(IDP->window)); /* remove the old graph window */
	IDP->window = NULL; /* flag no more graphing possible */
}

	setup_graph_dialog (bp);
}

void popup_graph (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Graph",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_graph,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_graph,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },
	{ "/Clear graph",	   NULL,	   clear_graph,		0,	  "<Item>" }

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_graph (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->xscale = get_prop_val (node, "xscale");
	IDP->xoffset = get_prop_val (node, "xoffset");
	IDP->yscale = get_prop_val (node, "yscale");
	IDP->yoffset = get_prop_val (node, "yoffset");
	make_graph (fp->bp);
}
void write_graph (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "xscale", IDP->xscale);
	set_prop_val (node, "xoffset", IDP->xoffset);
	set_prop_val (node, "yscale", IDP->yscale);
	set_prop_val (node, "yoffset", IDP->yoffset);
}
