/*
 *  The graphical definition element of deltaFSD
 *  fileops.c - read/write data files 
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */    
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include "fileops.h"
# include "function.h"
# include <stdio.h> 
# include <string.h> 
# include "engine.h"
# include "main.h" /* need to set main mode */

gdouble get_prop_val (xmlNodePtr node, xmlChar *name)
{
xmlChar* value;
gdouble f = 0.0;
	if ((value = (xmlGetProp (node, name)))) sscanf (value, "%lf", &f);
	return (f);
}

void set_prop_val (xmlNodePtr node, xmlChar *name, gdouble f)
{
xmlChar buffer[BUFFER_SIZE];
	snprintf (buffer, BUFFER_SIZE, "%lf", f);
	xmlSetProp(node, name, buffer);
}

boxp process_box (xmlNodePtr node, futs funct)
{
boxp bp;
funs fp;
gdouble x, y;
	x = get_prop_val (node, "x");
	y = get_prop_val (node, "y");
	bp = new_box (bpstart, x, y); /* bpstart holds the current module head */
	fp = function_insert (funct, bp); /* create the function instance and initialize it (if required) */
	if (funct->funcext && funct->funcext->fcreate) (funct->funcext->fcreate)(node, fp); /* do any per instance initialization */
	return (bp);
}

boxp file_box (xmlNodePtr node)
{
int i;
	for (i=0; functions[i].mentry != NULL; i++)
		if (functions[i].fname && !strcmp (functions[i].fname, node->name))
			return (process_box (node, &(functions[i])));
	return NULL; /* no match found */
}	

void file_line (xmlNodePtr parent, sstubp ssp, dstubp dsp)
{
lsp ls;
xmlNodePtr seg_node;
	seg_node = parent->childs;
	if (!seg_node) return; /* no children - no line to make */
	ls = new_line (ssp, get_prop_val(seg_node, "x"), get_prop_val(seg_node, "y"));
	for (seg_node = seg_node->next; seg_node != NULL; seg_node = seg_node->next)
	ls = new_seg (ls, get_prop_val(seg_node, "x"), get_prop_val(seg_node, "y"));
	ls->lp->dststub = dsp;
	complete_line (ls->lp);	
}

sstubp get_src_stub (boxp bp, int sstubno)
{
sstubp sp;
	if (!bp) return (NULL);
	for (sp = bp->ssbp; sp != NULL; sp = sp->next)
		if (sp->numb == sstubno) return (sp);
	return (NULL);
}

dstubp get_dst_stub (boxp bp, int dstubno)
{
dstubp dp;
	if (!bp) return (NULL);
	for (dp = bp->dsbp; dp != NULL; dp = dp->next)
		if (dp->numb == dstubno) return (dp);
	return (NULL);
}

void file_read (GtkWidget *w, GtkFileSelection *fs)
{
boxp bp[MAX_BOXES_PER_FILE]; /* the box cross referencing table fix */
/* moule by module numbering means we need this "local" cross reference array */
char *fname;
xmlDocPtr doc;
xmlNodePtr node;

	bp[0] = bpstart; /* box zero is the containing module */
    fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	if (fname) doc = xmlParseFile(fname); else return;
	if (!doc) return; /* need some error hanfdling - fix */
	/* we have a valid (?) filename so can remove the window */
	gtk_widget_destroy (w->parent->parent->parent);
	mmode = FILEREAD;
	for (node = doc->root->childs; node != NULL; node = node->next)
		if (strcmp (node->name, "Line"))
			bp[(int)get_prop_val(node, "reference")] = file_box (node);
		else
			{
			sstubp sp; dstubp dp;
			sp = get_src_stub (bp[(int)get_prop_val(node, "source_reference")],
				(int)get_prop_val(node, "source_stub"));
			dp = get_dst_stub (bp[(int)get_prop_val(node, "destination_reference")],
				(int)get_prop_val(node, "destination_stub"));
			if (sp && dp) file_line (node, sp, dp); /* cannot join boxes that don't exist */
			} 
	cursel.typ = NOTHING; /* reset the mode */
	mmode = SELECT;
	redraw_request ("File read");
	xmlFreeDoc (doc); /* clear the xml doc tree */
} 
void line_node (xmlNodePtr parent, linp lp)
{ /* outputs a single line */
xmlNodePtr node, seg_node;
lsp lseg;
    node = xmlNewChild(parent, NULL, "Line", NULL);
	set_prop_val(node, "source_reference", lp->srcstub->bp->bcount);
	set_prop_val(node, "source_stub", lp->srcstub->numb);
	set_prop_val(node, "destination_reference", lp->dststub->bp->bcount);
	set_prop_val(node, "destination_stub", lp->dststub->numb);
	
	for (lseg = lp->frstseg; lseg != NULL; lseg = lseg->next)
		{
		seg_node = xmlNewChild(node, NULL, "segment", NULL);
		set_prop_val(seg_node, "x", lseg->x);
		set_prop_val(seg_node, "y", lseg->y);
		}
}
char *box_node (xmlNodePtr parent, boxp bp)
{ /* outputs a single function */
xmlNodePtr node;	
    node = xmlNewChild(parent, NULL, bp->fp?bp->fp->funct->fname:"Module", NULL);
	set_prop_val(node, "reference", bp->bcount);
	set_prop_val(node, "x", bp->x);
	set_prop_val(node, "y", bp->y);
	if (bp->fp->funct->funcext && bp->fp->funct->funcext->fwrite)
		bp->fp->funct->funcext->fwrite(node, bp);
	return NULL;
}
void file_write (GtkWidget *w, GtkFileSelection *fs)
{
boxp bp;
dstubp dsbp;
char *fname;
xmlDocPtr doc;
int boxcount = 1; /* box zero is the containing module */
/* box counting is awkward - we only deal with modules so can't have program wide unique reference numbers. */
    doc = xmlNewDoc("1.0");
    doc->root = xmlNewDocNode(doc, NULL, "deltaFSD_data", NULL);
	if (bpstart) for (bp = bpstart->next; bp != NULL; bp = bp->next)
		{ // write all boxes but ignore the containing module */
		bp->bcount = boxcount++; /* tag the box with its id number */
		box_node (doc->root, bp);
		}
	for (bp = bpstart; bp != NULL; bp = bp->next) /* but process lines to the module */
		for (dsbp = bp->dsbp; dsbp != NULL; dsbp = dsbp->next)
			if (dsbp->lp) line_node (doc->root, dsbp->lp);
	// tree built => write it out to the file
    fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	if (-1 == xmlSaveFile (fname, doc)) puts ("Save failed");
	xmlFreeDoc (doc); /* clear the xml doc tree */
	gtk_widget_destroy (w->parent->parent->parent); /* remove the window */
}

void fileops (GtkWidget* w, gpointer action)
{
	file_select (action, NULL); /* action will be file_read or file_write */
}
