/*
 *  The graphical definition element of deltaFSD
 *  fmisc.c - utilities for functions - menus, definition, instantiation
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */	
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */



# include <stdlib.h>
# include <string.h>
# include "fmisc.h"
# include "function.h"
# include "main.h"
# include "deletion.h"

/* the ui utilities */
GtkWidget *create_entry_vbox (GtkSignalFunc callback, GtkObject *cdp) 
{ /* create a vbox with an OK button */
GtkWidget *button, *window;
// create window
	window = gtk_dialog_new ();
	gtk_widget_show (window);
// the action area contains a button
	button = gtk_button_new_with_label ("OK");
	gtk_widget_show (button);
	if (!cdp) cdp = (GtkObject *)window; /* if no paramter given use the window as default */
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked", callback, cdp);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), button, FALSE, FALSE, 10);
	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
	return (window);
}

GtkWidget *prompt_entry (GtkWidget *vbox, char *prompt, char *dfault)
{ /* add a prompt/entry pair to a vbox */
GtkWidget *hbox, *label, *entry;
	hbox = STD_HBOX;
// the hbox contains a label
	label = gtk_label_new (prompt);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
	STD_PACK (GTK_BOX (hbox), label);
// and an entry
	entry = gtk_entry_new_with_max_length (BUFFER_SIZE);
	gtk_entry_set_text (GTK_ENTRY (entry), dfault);
	STD_PACK (GTK_BOX (hbox), entry);
// pack the hbox into the vbox we were given
	STD_PACK (GTK_BOX (vbox), hbox);
	gtk_widget_show (label);
	gtk_widget_show (entry);
	gtk_widget_show (hbox);
	return (entry);
}

void text_info (GtkWidget *vbox, char *prompt)
{
GtkWidget *hbox, *label, *separator;
	hbox = STD_HBOX;
// the hbox contains a label
	label = gtk_label_new (prompt);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
	STD_PACK (GTK_BOX (hbox), label);
// pack the hbox into the vbox we were given
	STD_PACK (GTK_BOX (vbox), hbox);
// then a separator - is this required ??
	separator = gtk_hseparator_new ();
	STD_PACK (GTK_BOX (vbox), separator);
	gtk_widget_show (label);
	gtk_widget_show (separator);
	gtk_widget_show (hbox);
}

/* general purpose warning */
void warning_box (char *text)
{
GtkWidget *wind, *label;
	wind = create_entry_vbox (gtk_widget_destroy, NULL); /* data will default to window */
	label = gtk_label_new (text);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
	STD_PACK (GTK_BOX(GTK_DIALOG (wind)->vbox), label);
	gtk_widget_show (label);
}

/* standard file selection utility */
GtkWidget *file_select (GtkSignalFunc func, gpointer func_data)
{
GtkWidget *file_widget;
	/* Create a new file selection widget */
    file_widget = gtk_file_selection_new ("File selection");
	/* set up the default data */
	if (!func_data) func_data = file_widget;
	/* connect up the signals */
	gtk_signal_connect (GTK_OBJECT (file_widget), "destroy", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (file_widget));
    /* Connect the ok_button to file_ok_sel function */
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_widget)->ok_button), "clicked", func, func_data);
    /* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (file_widget)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (file_widget));
    /* No default filename  - ie with gtk_file_selection_set_filename */
    gtk_widget_show (file_widget);
    return (file_widget);
}

/* the registration and menu utilities */
GtkItemFactoryEntry *menu_items = NULL; /* these need to be global - they are build in add_menu_items */
gint nmenu_items = 0;	/* but used to create the item factory in get_main_menu */

void add_menu_item (gchar *path, gchar *accelerator, GtkItemFactoryCallback callback, guint callback_action, gchar *item_type)
{
GtkItemFactoryEntry item;
	item.path = path;
	item.accelerator = accelerator;
	item.callback = callback;
	item.callback_action = callback_action;
	item.item_type = item_type;
	menu_items = realloc (menu_items, (nmenu_items+1)*sizeof (GtkItemFactoryEntry));
	memcpy (menu_items+nmenu_items, &item, sizeof(item));
	nmenu_items++;
}
	
/* popup utilities */
void standard_popup (boxp bp)
{
static char buffer[BUFFER_SIZE] = "/";
static GtkItemFactoryEntry popup_items[] = {
	{ buffer,			NULL,	NULL,				0,	"<Item>" },
	{ "/",				NULL,	NULL,				0,	"<Separator>" },
	{ "/Delete",		NULL,	popup_delete_box,	0,	"<Item>" },	
	{ "/Force update",	NULL,	force_update,		0,	"<Item>" }};	
	/* copy in the parameter */
	strncpy (buffer+1, bp->fp->funct->fname, BUFFER_SIZE-1);	
	popup_items[2].callback_action = (guint) bp;
	popup_items[3].callback_action = (guint) bp;
	popup_box (popup_items, bp->x, bp->y, sizeof (popup_items) / sizeof (popup_items[0]));
}
/* line popup */
void popup_delete_line (GtkWidget* w, linp lp)
{
	delete_line (lp);
	redraw_request ("Line delete");
}

void line_popup (linp lp, gdouble x, gdouble y)
{
static GtkItemFactoryEntry popup_items[] = {
	{ "/Line",			NULL,		NULL,				0,	"<Item>" },
	{ "/",				NULL,		NULL,				0,	"<Separator>" },
	{ "/Delete",		NULL,		popup_delete_line,	0,	"<Item>" },	
	{ "/Force update",	NULL,		force_update,		0,	"<Item>" }
	};	
	/* copy in the parameter */
	popup_items[2].callback_action = (guint) lp;
	popup_items[3].callback_action = (guint) lp->srcstub->bp;
	popup_box (popup_items, x, y, sizeof (popup_items) / sizeof (popup_items[0]));
}

void popup_box (GtkItemFactoryEntry *popup_items, guint winx, guint winy, int nitems)
{ 
gint x, y;
GtkItemFactory *item_factory;
	/* no accel group here ? */
	item_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
	gtk_item_factory_create_items (item_factory, nitems, popup_items, NULL);
	/* the pop up requires absolute x and y - we need to know where the main window is */
	gdk_window_get_position (root_window.main_vbox->window, &x, &y);
	gtk_item_factory_popup (item_factory, x + winx, y + winy, 2, 1); /* mouse button 2, time 1 */
}
	
/* standard box and function delete on pop up option menu */
void popup_delete_box (GtkWidget* w, boxp bp)
{
	delete_box (bp);
	redraw_request ("Box delete");
}

/* redraw utilities */
void update_and_redraw (funs fp)
{
	process_func (fp);
	redraw_request ("Function update");
}
void force_update (GtkWidget* w, boxp bp)
{ /* called from the popup menus */
	update_and_redraw (bp->fp);
}

/* the menu */
void quit () { gtk_exit (0); }

void help_message ()
{
GtkWidget *window;
	window = create_entry_vbox (gtk_widget_destroy, NULL);
	gtk_widget_show (window);
	text_info ((GTK_DIALOG (window)->vbox), VERSION_STRING);
}

void get_main_menu(GtkWindow *window, GtkWidget **menubar )
{
GtkItemFactory *item_factory;
GtkAccelGroup *accel_group;

	add_menu_item ("/_File", NULL, NULL, 0, "<Branch>");
	add_menu_item ("/File/Read file", NULL, fileops, (int)file_read, NULL);
	add_menu_item ("/File/Write file", NULL, fileops, (int)file_write, NULL);
	add_menu_item ("/File/sep1",	NULL, NULL, 0, "<Separator>");
	add_menu_item ("/File/Quit", NULL, quit, 0, NULL);
	register_functions ();
	add_menu_item ("/_Help", NULL, NULL, 0, "<LastBranch>");
	add_menu_item ("/_Help/About", NULL, help_message, 0, NULL);

	accel_group = gtk_accel_group_new ();
	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
	gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
	
/* Attach the new accelerator group to the window. */
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
	if (menubar) /* Finally, return the actual menu bar created by the item factory. */ 
		*menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}
