
/*
 *	The graphical definition element of deltaFSD
 *	fmodule.c - the various components of the module pseudo-function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_module (futs f);
void instance_module (funs fp);
void popup_module (boxp bp);
void create_module (xmlNodePtr node, funs fp);
void write_module (xmlNodePtr node, boxp bp);
void remove_module (funs fp);


struct Fx module_functions = {
	init_module,			/* when the module function type is created */
	instance_module,		/* create instance from UI */
	popup_module,		/* right click on hotspot */
	create_module,		/* create instance from file */
	write_module,		/* generate text version of configuration */
	remove_module,		/* any per instance clean up */
	"Modules are used as containers for collections of functions\nSet the number of inputs and outputs you need\n(Use the hot spot in the top left corner of the box\nto descend into or ascend out of any module)"
	};

/* the module "super-box" */
# define MODBOX_X STUBLEN
# define MODBOX_Y STUBLEN
# define MODBOX_H (MAIN_WINDOWH-2*STUBLEN)
# define MODBOX_W (MAIN_WINDOWW-2*STUBLEN)


typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*ins_entry;
	GtkWidget	*outs_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	GtkWidget *	emesg;
	int	outs;
	int	ins;
	} instance_data;
	
void popup_ascdesc_module (gchar *string, boxp bp);


/* Module function */
void Module (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{ /* not executed but needed to anchor inputs and outputs */
}


void popup_ascdesc_module (gchar *string, boxp bp)
{
        if (bp) bpstart = bp; /* the parameter will be the new module list head */
        redraw_request ("Module level change");
}




void make_module (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the module start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
int i;
boxp child_box;
        IDP->fp->ins = IDP->ins; /* update the input and output count */
        IDP->fp->outs = IDP->outs;
        add_funcio (IDP->fp); /* the parent box exists */
        /* now create the child box */
        child_box = create_box (MODBOX_X, MODBOX_Y, MODBOX_W, MODBOX_H);
        child_box->fp = IDP->fp; /* copy across the function */
        child_box->prnt = IDP->fp->bp;
        child_box->text1 = IDP->text1;
        child_box->text2 = IDP->text2;
        child_box->text3 = IDP->text3;
        IDP->fp->bp->chld = child_box; /* link it in */
        /* and create its stubs */
        for (i = 0; i < IDP->ins; i++) new_srcstub (child_box, i, NULL);
        for (i = 0; i < IDP->outs; i++) new_dststub (child_box, i, NULL);
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void module_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->ins_entry));
	sscanf(entry_text, "%d", &(IDP->ins));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->outs_entry));
	sscanf(entry_text, "%d", &(IDP->outs));
	gtk_widget_destroy (IDP->dwindow);
	make_module (bp);
}

void init_module (futs f)
{
}

void setup_module_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (module_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->ins_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of inputs", "1");
	IDP->outs_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of outputs", "1");
}
void instance_module (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_module_dialog (fp->bp);
}

void remove_module (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
	if (fp->bp->chld->next)     /* the child has some contained boxes - can't delete */
			{
			warning_box ("The module is not empty - cannot delete");
			return; /* nothing else to do */
			}
	if (fp->bp->mod_top) bpstart = fp->bp->mod_top; /* ensure we go up to the parent level */
	delete_box (fp->bp->chld); /* remove the child and any lines it has */
	/* the standard remove will destroy the parent */
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Module destroyed");
}

void popup_delete_module (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_module (bp->fp);
}
void popup_adjust_module (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
	return; /* don't process */
}

	setup_module_dialog (bp);
}

void popup_module (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Module",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_module,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_module,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },
	{ "/Ascend",	NULL,		popup_ascdesc_module,	0, "<Item>" },
	{ "/Descend",   NULL,       popup_ascdesc_module,	0, "<Item>" },

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;
{
int i, item_count;
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint)(bp->prnt ? bp->prnt : bp);     /* pass the parent */

	popup_items[5].item_type = popup_items[6].item_type     = "<Title>"; /* assume no child or parent */
	if (bp->prnt && bp->prnt->mod_top) /* can ascend */
		{
		popup_items[5].item_type = "<Item>"; /* ascend can be clicked */
		popup_items[5].callback_action = (guint) bp->prnt->mod_top; /* where to go if we are clicked */
		}
	if (bp->chld) /* can descend */
		{
		popup_items[6].item_type = "<Item>"; /* descend can be clicked */
		popup_items[6].callback_action = (guint) bp->chld; /* where to go if we are clicked */
		}
}

	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_module (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->ins = get_prop_val (node, "ins");
	IDP->outs = get_prop_val (node, "outs");
	make_module (fp->bp);
}
void write_module (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "ins", IDP->ins);
	set_prop_val (node, "outs", IDP->outs);
}
