
/*
 *	The graphical definition element of deltaFSD
 *	fperl.c - the various components of the perl interface function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"

# include <EXTERN.h>
# include <perl.h>

/* we only have one (global) perl interpreter at any time */
/* the name space issue means that we can have only have one (top level) file */
/* we ease this restriction (a little) by using the dynaloader */
PerlInterpreter *interp = NULL;


void init_perl (futs f);
void instance_perl (funs fp);
void popup_perl (boxp bp);
void create_perl (xmlNodePtr node, funs fp);
void write_perl (xmlNodePtr node, boxp bp);
void remove_perl (funs fp);


struct Fx perl_functions = {
	init_perl,			/* when the perl function type is created */
	instance_perl,		/* create instance from UI */
	popup_perl,		/* right click on hotspot */
	create_perl,		/* create instance from file */
	write_perl,		/* generate text version of configuration */
	remove_perl,		/* any per instance clean up */
	"Calls the named Perl sub from the specified file\nSet the number of inputs and outputs your sub needs"
	};



typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*ins_entry;
	GtkWidget	*outs_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	GtkWidget *	pfile;
	path_buffer	fpath;
	int	outs;
	int	ins;
	} instance_data;
	
extern void boot_DynaLoader (CV* cv);
void setup_perl_dialog (boxp bp);
void destroy_interpreter (GtkWidget* w, boxp bp);
void setup_perl_source (GtkWidget *w, instance_data *idp);


/* Perl function */
void Perl (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{
int i, count;
	dSP;                            /* initialize stack pointer */
	ENTER;                          /* everything created after here */
	SAVETMPS;                       /* ...is a temporary variable. */
	PUSHMARK (SP);          /* remember the stack pointer */
	/* push the parameters onto the stack */
	for (i = 0; i < fp->ins; i++) XPUSHs (sv_2mortal(newSVnv(In(i))));
	PUTBACK;                        /* make local stack pointer global */
	count = call_pv(fp->bp->text2, G_ARRAY | G_EVAL);        /* call the function */
	SPAGAIN;                        /* refresh stack pointer */
	if (count > fp->outs) count = fp->outs; /* the minimum of both */
	/* pop the return values from stack in reverse order */
	for (i = count - 1; i > -1; i--) Out(i) = POPn;
	PUTBACK;
	FREETMPS;                       /* free that return value */
	LEAVE;     
}


void xs_init (void)
{
	char *file = __FILE__;
	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
}

void init_interpreter (char *fname)
{ /* loads the global interp */
char *args[2] = {"", ""};
	// extract the file name
	args[1] = fname;
	// invoke the interpreter
	interp = perl_alloc();
	perl_construct (interp);
	perl_parse (interp, xs_init, 2, args, NULL);
	perl_run (interp);
}

void setup_perl_source (GtkWidget *w, instance_data *idp)
{ /* grab the file name and feed it to the interpreter */
	strncpy (idp->fpath, gtk_file_selection_get_filename (GTK_FILE_SELECTION (idp->pfile)), PATH_BUFFER_SIZE);
	gtk_widget_destroy (idp->pfile); /* destroy the file selection window */
	setup_perl_dialog (idp->fp->bp);
}

void destroy_interpreter (GtkWidget* w, boxp bp)
{
	if (!interp) return;
	perl_destruct(interp);
    perl_free(interp);
	interp = NULL; /* rady for the next one */
	delete_box (bp); /* remove the box */
	redraw_request ("Perl deleted");
}




void make_perl (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the perl start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	IDP->fp->bp->text1 = IDP->fp->funct->fname; /* reinstate the name */
	IDP->fp->ins = IDP->ins; /* copy in the parameters */
	IDP->fp->outs = IDP->outs;
	add_funcio (IDP->fp); /* this will have already been called but with no inputs or outputs */
// start up the interpreter - if required
	if (!interp) init_interpreter (IDP->fpath); /* interpret the file */
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void perl_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->ins_entry));
	sscanf(entry_text, "%d", &(IDP->ins));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->outs_entry));
	sscanf(entry_text, "%d", &(IDP->outs));
	gtk_widget_destroy (IDP->dwindow);
	make_perl (bp);
}

void init_perl (futs f)
{
}

void setup_perl_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (perl_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Sub name", "?");
	IDP->ins_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of inputs", "1");
	IDP->outs_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of outputs", "1");
}
void instance_perl (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
	if (!interp) /* need to initialize one and load in the top level file */
		{
		IDP->pfile =  file_select (setup_perl_source, IDP);
		return; /* we'll get to setup_perl_dialog via the callaback from setup_perl_source */
		}
}

	setup_perl_dialog (fp->bp);
}

void remove_perl (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Perl destroyed");
}

void popup_delete_perl (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_perl (bp->fp);
}
void popup_adjust_perl (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
}

	setup_perl_dialog (bp);
}

void popup_perl (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Perl",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_perl,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_perl,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },
	{ "/Destroy interpreter",       NULL,   destroy_interpreter,    0,      "<Item>" }

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_perl (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->ins = get_prop_val (node, "ins");
	IDP->outs = get_prop_val (node, "outs");
	strncpy (IDP->fpath, xmlGetProp (node, "fpath"), sizeof(IDP->fpath));
	make_perl (fp->bp);
}
void write_perl (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "ins", IDP->ins);
	set_prop_val (node, "outs", IDP->outs);
	xmlSetProp(node, "fpath", IDP->fpath);
}
