/*
 *  The graphical definition element of deltaFSD
 *  fsandh.c - the various components of the two sample and hold functions
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */    
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */

/*
 * The two sampling functions:
 *	i)	Track and Hold
 *	ii)	Sample on rising edge
 */

# include <stdlib.h>
# include "function.h"
# include "main.h"

struct Fx trackhold_functions =
	{
	NULL,					// when the type of function is created
	instance_trackhold,			// create instance from UI
	NULL,					// called on right click
	NULL,			// create instance from file
	NULL,				// genrate text version of configuration
	NULL, 					// any per instance clean up 
	"When the second input to the sample and hold is\n"	// the descriptive text
	"non-zero the output will follow the first input.\n"
	"When the second input is zero the output is frozen."
};

typedef struct {
	GtkWidget *dwindow;
	funs fp;
} trackhold_data;

void TrackHold (funs fp) 
{ 
	if (In(1)) Out(0) = In(0);
}	


void trackhold_button_press (trackhold_data* tdp)
{
	gtk_widget_destroy (tdp->dwindow);
	free (tdp);
}

void setup_trackhold_dialog (trackhold_data *tdp)
{
	tdp->dwindow = create_entry_vbox (trackhold_button_press, (GtkObject*)tdp); /* cast ?? */		// the vbox of the dialogue contains two prompt entry pairs
// add the caption
	text_info ((GTK_DIALOG (tdp->dwindow)->vbox), tdp->fp->funct->funcext->describe);
// the vbox of the dialogue contains no prompt entry pairs
}	

void instance_trackhold (funs f)
{
trackhold_data *tdp;
	tdp = malloc (sizeof (*tdp));  /* create the data area */
	tdp->fp = f;
	setup_trackhold_dialog (tdp);
}

struct Fx edgesample_functions =
	{
	NULL,					// when the type of function is created
	instance_edgesample,			// create instance from UI
	NULL,					// called on right click
	NULL,			// create instance from file
	NULL,				// genrate text version of configuration
	NULL, 					// any per instance clean up 
	"When the second input goes from zero to non-zero\n"	// the descriptive text
	"the first output is updated to match the first input.\n"
	"The second output is the logical (0/1) equivalent of the second input."
};

typedef struct {
	GtkWidget *dwindow;
	funs fp;
} edgesample_data;


void EdgeSample (funs fp) 
{
	if (In(1) && !Out(1)) Out(0) = In(0);
	Out(1) = In(1) ? 1.0 : 0.0;
}	

	
void edgesample_button_press (edgesample_data* edp)
{
	gtk_widget_destroy (edp->dwindow);
	free (edp);
}

void setup_edgesample_dialog (edgesample_data *edp)
{
	edp->dwindow = create_entry_vbox (edgesample_button_press, (GtkObject*)edp); /* cast ?? */		// the vbox of the dialogue contains two prompt entry pairs
// add the caption
	text_info ((GTK_DIALOG (edp->dwindow)->vbox), edp->fp->funct->funcext->describe);
// the vbox of the dialogue contains no prompt entry pairs
}	

void instance_edgesample (funs f)
{
edgesample_data *edp;
	edp = malloc (sizeof (*edp));  /* create the data area */
	edp->fp = f;
	setup_edgesample_dialog (edp);
}
