
/*
 *	The graphical definition element of deltaFSD
 *	fsocket.c - the various components of the socket communications function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>


void init_socket (futs f);
void instance_socket (funs fp);
void popup_socket (boxp bp);
void create_socket (xmlNodePtr node, funs fp);
void write_socket (xmlNodePtr node, boxp bp);
void remove_socket (funs fp);


struct Fx socket_functions = {
	init_socket,			/* when the socket function type is created */
	instance_socket,		/* create instance from UI */
	popup_socket,		/* right click on hotspot */
	create_socket,		/* create instance from file */
	write_socket,		/* generate text version of configuration */
	remove_socket,		/* any per instance clean up */
	"This function provides a socket interface\nSet the port number for stream input/output\nand the number of inputs and outputs for the function"
	};

# define DATA_BUFFER_SIZE 200 /* enough for about ten channels ?? */


typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*ins_entry;
	GtkWidget	*outs_entry;
	GtkWidget	*port_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	int	list_tag;
	int	peer_fd;
	int	port;
	int	outs;
	int	ins;
	int	list_fd;
	int	peer_tag;
	} instance_data;
	
void accept_and_read (instance_data *idp, gint source, GdkInputCondition condition);
void listen_and_accept (instance_data *idp);
void close_connection (instance_data *idp);


/* Socket function */
void Socket (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{
char buf[DATA_BUFFER_SIZE]; 
int i;
char LF = '\n'; /* new line */
	if (IDP->peer_fd == 0) return; /* no fd to send via */
	for (i=0; i<fp->ins; i++)
		{
		snprintf (buf, DATA_BUFFER_SIZE, "%d,%f,", i, In(i));
		if (write(IDP->peer_fd, buf, strlen(buf)) < strlen(buf)) break;
		}	
	if (write(IDP->peer_fd, &LF, 1) < 1) close_connection (IDP);
}


void set_channels (instance_data *idp, char *strtptr)
{
int chan;
double val;
char *endptr;

	for (;;)
		{
		if (!*strtptr) return;
		chan = strtol (strtptr, &endptr, 10);
		if (strtptr == endptr) return;
		val = strtod (endptr+1, &strtptr);
		if (strtptr == endptr+1) return;
		if (chan < idp->fp->outs) (idp->fp->frstout[chan]).next_value = val;
		if (!*strtptr++) return;
		}
}
void close_connection (instance_data *idp)
{
	close (idp->peer_fd);
	gdk_input_remove( idp->peer_tag );
	idp->peer_fd = 0; /* block any further sends */
}
void read_all_data (instance_data *idp, gint source, GdkInputCondition condition)
{
char buf[DATA_BUFFER_SIZE];
int bcount;

	bcount = read (idp->peer_fd, buf, DATA_BUFFER_SIZE);
	if (bcount < 1 || (condition & GDK_INPUT_EXCEPTION) || idp->peer_fd < 1) 
		close_connection (idp);
	else
		{
		buf[bcount] = 0; /* terminate the string */
		set_channels (idp, buf);
		update_and_redraw (idp->fp); /* will force recalc - and so output - fix ?? */
		}
}

void accept_and_read (instance_data *idp, gint source, GdkInputCondition condition)
{  // dispparing peer creates reads with no data - NOT an EXCEPTION condition 
struct sockaddr_in peer_sa;
int addr_len;

	addr_len = sizeof (peer_sa);
	idp->peer_fd = accept (idp->list_fd, (struct sockaddr *) &peer_sa, &addr_len);
	if (idp->peer_fd == 0 || (condition & GDK_INPUT_EXCEPTION) || idp->list_fd < 1 ) 
		{
		close (idp->list_fd);
		idp->list_fd = 0;
		gdk_input_remove (idp->list_tag);
		puts ("Connection socket failed");
		listen_and_accept (idp); /* restart */
		return;
		}
	idp->peer_tag = gdk_input_add(idp->peer_fd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, (GdkInputFunction)read_all_data, (gpointer)idp);
}

void listen_and_accept (instance_data *idp)
{
struct sockaddr_in list_sa;
int one = 1;

    idp->list_fd = socket (AF_INET, SOCK_STREAM, 0);

    setsockopt (idp->list_fd, SOL_SOCKET, SO_REUSEADDR,  (char *) &one, sizeof(one));
    list_sa.sin_addr.s_addr = INADDR_ANY;
    list_sa.sin_port = htons ((short)idp->port);
    list_sa.sin_family = AF_INET;
    bind (idp->list_fd, (struct sockaddr *) &list_sa, sizeof (list_sa));
	/* connection set up - start listening */
    listen (idp->list_fd, 1); /* no queue */
	/* everything is ready - set up the callback */
    idp->list_tag = gdk_input_add(idp->list_fd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, (GdkInputFunction)accept_and_read, (gpointer)idp);
}




void make_socket (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the socket start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	IDP->fp->ins = IDP->ins; /* copy in the parameters */
	IDP->fp->outs = IDP->outs;
	add_funcio (IDP->fp); /* this will have already been called but with no inputs or outputs */
	listen_and_accept (IDP);
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void socket_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->ins_entry));
	sscanf(entry_text, "%d", &(IDP->ins));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->outs_entry));
	sscanf(entry_text, "%d", &(IDP->outs));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->port_entry));
	sscanf(entry_text, "%d", &(IDP->port));
	gtk_widget_destroy (IDP->dwindow);
	make_socket (bp);
}

void init_socket (futs f)
{
}

void setup_socket_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (socket_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->ins_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of inputs", "1");
	IDP->outs_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Number of outputs", "1");
	IDP->port_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Port number", "6739");
}
void instance_socket (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_socket_dialog (fp->bp);
}

void remove_socket (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{
		if (IDP->peer_tag) gdk_input_remove (IDP->peer_tag);
		if (IDP->list_tag) gdk_input_remove (IDP->list_tag);
		IDP->peer_tag = IDP->list_tag = 0;
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Socket destroyed");
}

void popup_delete_socket (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_socket (bp->fp);
}
void popup_adjust_socket (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{
		if (IDP->peer_tag) gdk_input_remove (IDP->peer_tag);
		if (IDP->list_tag) gdk_input_remove (IDP->list_tag);
		IDP->peer_tag = IDP->list_tag = 0;
}

	setup_socket_dialog (bp);
}

void popup_socket (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Socket",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_socket,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_socket,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_socket (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->ins = get_prop_val (node, "ins");
	IDP->outs = get_prop_val (node, "outs");
	IDP->port = get_prop_val (node, "port");
	make_socket (fp->bp);
}
void write_socket (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "ins", IDP->ins);
	set_prop_val (node, "outs", IDP->outs);
	set_prop_val (node, "port", IDP->port);
}
