
/*
 *	The graphical definition element of deltaFSD
 *	fticker.c - the various components of the ticker function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_ticker (futs f);
void instance_ticker (funs fp);
void popup_ticker (boxp bp);
void create_ticker (xmlNodePtr node, funs fp);
void write_ticker (xmlNodePtr node, boxp bp);
void remove_ticker (funs fp);


struct Fx ticker_functions = {
	init_ticker,			/* when the ticker function type is created */
	instance_ticker,		/* create instance from UI */
	popup_ticker,		/* right click on hotspot */
	create_ticker,		/* create instance from file */
	write_ticker,		/* generate text version of configuration */
	remove_ticker,		/* any per instance clean up */
	"The ticker's output is a 0-1-0 pulse train\nThe rate setting determines how often the output toggles between 0.0 and 1.0\nThe input is used to reset the ticker (when non-zero)"
	};



typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*rate_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	int	rate_value;
	int	timeout_id;
	int	prev_state;
	} instance_data;
	
int ticker_callback (instance_data* idp);


/* Ticker function */
void Ticker (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{
	if (!In(0)) IDP->prev_state = 1; /* force prev to 1 - hence cur to 0 */
	Out(0) = IDP->prev_state = !IDP->prev_state;
}


int ticker_callback (instance_data* idp)
{ /* time out callback */
	idp->fp->frstout->next_value = idp->fp->frstout->next_value + idp->rate_value/1000.0;
	update_and_redraw (idp->fp);
	return 1;
}




void make_ticker (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the ticker start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	IDP->timeout_id = gtk_timeout_add (IDP->rate_value, (GtkFunction)ticker_callback, IDP);
	IDP->fp->frstout->next_value = 0.0; /* initailize the ticker */
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void ticker_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->rate_entry));
	sscanf(entry_text, "%d", &(IDP->rate_value));
	gtk_widget_destroy (IDP->dwindow);
	make_ticker (bp);
}

void init_ticker (futs f)
{
}

void setup_ticker_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (ticker_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->rate_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Rate", "100");
}
void instance_ticker (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_ticker_dialog (fp->bp);
}

void remove_ticker (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{	/* stop the timer */
	gtk_timeout_remove (IDP->timeout_id);
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Ticker destroyed");
}

void popup_delete_ticker (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_ticker (bp->fp);
}
void popup_adjust_ticker (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{	/* stop the timer */
	gtk_timeout_remove (IDP->timeout_id);
}

	setup_ticker_dialog (bp);
}

void popup_ticker (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Ticker",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_ticker,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_ticker,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_ticker (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->rate_value = get_prop_val (node, "rate");
	make_ticker (fp->bp);
}
void write_ticker (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "rate", IDP->rate_value);
}
