
/*
 *	The graphical definition element of deltaFSD
 *	ftimer.c - the various components of the timer function
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */


# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "function.h"
# include "main.h"
# include "deletion.h"
# include "engine.h"



void init_timer (futs f);
void instance_timer (funs fp);
void popup_timer (boxp bp);
void create_timer (xmlNodePtr node, funs fp);
void write_timer (xmlNodePtr node, boxp bp);
void remove_timer (funs fp);


struct Fx timer_functions = {
	init_timer,			/* when the timer function type is created */
	instance_timer,		/* create instance from UI */
	popup_timer,		/* right click on hotspot */
	create_timer,		/* create instance from file */
	write_timer,		/* generate text version of configuration */
	remove_timer,		/* any per instance clean up */
	"The output is the time (seconds) since the timer was reset\nThe rate setting determines how often the output is updated\nThe input is used to reset the timer (when non-zero)"
	};



typedef struct {
	funs fp;
	GtkWidget	*dwindow; /* the setup dialog window */
	GtkWidget	*name_entry;
	GtkWidget	*rate_entry;
	std_buffer	text1;
	std_buffer	text2;
	std_buffer	text3;
	int	rate_value;
	int	timeout_id;
	} instance_data;
	
int timer_callback(boxp bp);


/* Timer function */
void Timer (funs fp) 
# define IDP ((instance_data *)fp->bp->idata)
{
	if (!In(0)) Out(0) = 0.0;
	snprintf (IDP->text3, BUFFER_SIZE, "= %.4f", Out(0));
}


int timer_callback(boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* time out callback */
	bp->fp->frstout->next_value = bp->fp->frstout->next_value + IDP->rate_value/1000.0;
	update_and_redraw (bp->fp);
	return 1;
}




void make_timer (boxp bp)
# define IDP ((instance_data *)bp->idata)
{ /* the timer start code goes here */
	bp->text2 = IDP->text2;
	bp->text3 = IDP->text3;
{
	IDP->timeout_id = gtk_timeout_add (IDP->rate_value, (GtkFunction)timer_callback, bp);
	bp->fp->frstout->next_value = 0.0; /* initialize the timer */
}

	update_and_redraw (bp->fp); /* always recalc and redraw */
}

void timer_button_press (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->name_entry));
	strncpy (IDP->text2, entry_text, sizeof(IDP->text2));
	entry_text = gtk_entry_get_text(GTK_ENTRY(IDP->rate_entry));
	sscanf(entry_text, "%d", &(IDP->rate_value));
	gtk_widget_destroy (IDP->dwindow);
	make_timer (bp);
}

void init_timer (futs f)
{
}

void setup_timer_dialog (boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	IDP->dwindow = create_entry_vbox (timer_button_press, (GtkObject*)bp);
	text_info ((GTK_DIALOG (IDP->dwindow)->vbox), bp->fp->funct->funcext->describe);
	IDP->name_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Name", "?");
	IDP->rate_entry = prompt_entry ((GTK_DIALOG (IDP->dwindow)->vbox), "Rate", "100");
}
void instance_timer (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
{
}

	setup_timer_dialog (fp->bp);
}

void remove_timer (funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
{	/* stop the timer */
	gtk_timeout_remove (IDP->timeout_id);
}

	free (IDP);
	delete_box (fp->bp);
	redraw_request ("Timer destroyed");
}

void popup_delete_timer (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	remove_timer (bp->fp);
}
void popup_adjust_timer (GtkWidget* w, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
{	/* stop the timer */
	gtk_timeout_remove (IDP->timeout_id);
}

	setup_timer_dialog (bp);
}

void popup_timer (boxp bp)
{
int item_count, i;
static GtkItemFactoryEntry popup_items[] = {
	{ "/Timer",			NULL,		NULL,					0,	"<Item>" },
	{ "/",				NULL,		NULL,					0,	"<Separator>" },
	{ "/Modify",		NULL,		popup_adjust_timer,	0,	"<Item>" },	
	{ "/Delete",		NULL,		popup_delete_timer,	0,	"<Item>" },
	{ "/Force update",  NULL,		force_update,			0,	"<Item>" },

	};  
	item_count =  sizeof (popup_items) / sizeof (popup_items[0]);
	for (i=0; i<item_count; i++) /* copy in the parameter */
		popup_items[i].callback_action = (guint) bp;


	popup_box (popup_items, bp->x, bp->y, item_count);	
}

void create_timer (xmlNodePtr node, funs fp)
# define IDP ((instance_data *)fp->bp->idata)
{
	IDP = calloc (1, sizeof (instance_data));  /* create the data area */
	IDP->fp = fp; /* link to function */
	strncpy (IDP->text2, xmlGetProp (node, "name"), sizeof(IDP->text2));
	IDP->rate_value = get_prop_val (node, "rate");
	make_timer (fp->bp);
}
void write_timer (xmlNodePtr node, boxp bp)
# define IDP ((instance_data *)bp->idata)
{
	xmlSetProp(node, "name", IDP->text2);
	set_prop_val (node, "rate", IDP->rate_value);
}
