/*
 *  The graphical definition element of deltaFSD
 *  function.c - function definition
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */	
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */

# include <math.h>
# include <string.h>
# include "function.h"
# include "main.h"

/* Define the straightforward functions */
/* Math functions */
void Add (funs fp) { Out(0) = In(0) + In(1); }
void Subtract (funs fp) { Out(0) = In(0) - In(1); }
void Multiply (funs fp) { Out(0) = In(0) * In(1); }
void Divide (funs fp) { if (In(1)) Out(0) = In(0) / In(1); }
void Absolute (funs fp) {Out(0) = (In(0) > 0.0 ? In(0) : -In(0));}
void DivMod (funs fp) { if (In(1) != 0.0) {Out(0) = ((int)In(0)/(int)In(1)); Out(1) = ((int)In(0)%(int)In(1));}}
void IntFract (funs fp) {Out(0) = (int)In(0); Out(1) = In(0) - Out(0);}
void CeilFloor (funs fp) {Out(0) = ceil(In(0)); Out(1) = floor(In(0));}
void ExpLog (funs fp) { Out(0) = exp(In(0)); Out(1) = log(In(0)); }
void Power (funs fp) { Out(0) = pow(In(0), In(1)); }
/* Trig functions */
void Sin_math (funs fp) { Out(0) = sin (In(0));}
void Cos_math (funs fp) { Out(0) = cos (In(0));}
void Tan_math (funs fp) { Out(0) = tan (In(0));}
void Difference (funs fp) { Strobe(2); Out(0) = Out(0)+(In(1)*(In(0)-Out(0)));}
void Integral (funs fp) { Strobe(2); Out(0) = Out(0)+In(0)*In(1);}
/* Comparison functions */
void Compare (funs fp) {Out(0) = In(0) >= In(1) ? 1.0 : 0.0;}
void MinMax (funs fp) {Out(0) = In(0) >= In(1) ? In(0) : In(1); Out(1) = In(0) >= In(1) ? In(1) : In(0);}
void Hysteresis (funs fp) {Out(0) = fabs(In(0)) >= fabs(In (1)) ? In(0) : 0.0;}
/* Logic functions */
void And_logic (funs fp) { Out(0) = In(0) && In(1); }
void Or_logic (funs fp) { Out(0) = In(0) || In(1); }
void Xor_logic (funs fp) { Out(0) = (In(0) || In(1)) && !(In(0) && In(1)); }
void Not_logic (funs fp) { Out(0) = !In(0); }
void Bistable_logic (funs fp) { if (In(0)) Out(0) = Out(0) ? 0.0 : 1.0; }
void RSFF_logic (funs fp) { if (In(1)) Out(0) = In(0) ? 0.0 : 1.0; }
/* Utility functions */
void Delay (funs fp) { Out(0) = Out(1); Out(1) = In(0); }
/* The i/o and other utility functions get files to themselves */ 

struct Ft functions [] =
{
{ "/Math functions" },
{ "/Math functions/Add", "Add", Add, NULL, 2, 1 },
{ "/Math functions/Subtract", "Subtract", Subtract, NULL, 2, 1 },
{ "/Math functions/Multiply", "Multiply", Multiply, NULL, 2, 1 },
{ "/Math functions/Divide", "Divide", Divide, NULL, 2, 1 },
{ "/Math functions/Absolute", "Absolute", Absolute, NULL, 1, 1 },
{ "/Math functions/Div-Mod", "Div-Mod", DivMod, NULL, 2, 2 },
{ "/Math functions/Int-Fract", "Int-Fract", IntFract, NULL, 1, 2 },
{ "/Math functions/Ceil-Floor", "Ceil-Floor", CeilFloor, NULL, 1, 2 },
{ "/Math functions/Exp-Log", "Exp-Log", ExpLog, NULL, 1, 2 },
{ "/Math functions/Power", "Power", Power, NULL, 2, 1 },
{ "/Trig & Calc functions" },
{ "/Trig & Calc functions/Sin", "Sin", Sin_math, NULL, 1, 1 },
{ "/Trig & Calc functions/Cos", "Cos", Cos_math, NULL, 1, 1 },
{ "/Trig & Calc functions/Tan", "Tan", Tan_math, NULL, 1, 1 },
{ "/Trig & Calc functions/Integral", "Integral", Integral, NULL, 3, 1 },
{ "/Trig & Calc functions/Difference", "Difference", Difference, NULL, 3, 1 },
{ "/Comparison functions" },
{ "/Comparison functions/Compare", "Compare", Compare, NULL, 2, 1 },
{ "/Comparison functions/Min-Max", "Min-Max", MinMax, NULL, 2, 2 },
{ "/Comparison functions/Hysteresis", "Hysteresis", Hysteresis, NULL, 2, 1 },
{ "/Logic functions" },
{ "/Logic functions/And", "And", And_logic, NULL, 2, 1 },
{ "/Logic functions/Or", "Or", Or_logic, NULL, 2, 1 },
{ "/Logic functions/Xor", "Xor", Xor_logic, NULL, 2, 1 },
{ "/Logic functions/Not", "Not", Not_logic, NULL, 1, 1 },
{ "/Logic functions/RS-FlipFlop", "RS-FlipFlop", RSFF_logic, NULL, 2, 1 },
{ "/Logic functions/Bistable", "Bistable", Bistable_logic, NULL, 1, 1 },
{ "/Utility functions" },
{ "/Utility functions/Constant", "Constant", Constant, &const_functions, 0, 1 },
{ "/Utility functions/Delay", "Delay", Delay, NULL, 1, 2 },
{ "/Utility functions/Filter", "Filter", Filter, &filter_functions, 2, 1 },
{ "/Utility functions/Counter", "Counter", Counter, &counter_functions, 3, 1 },
{ "/Utility functions/Timer", "Timer", Timer, &timer_functions, 1, 1 },
{ "/Utility functions/Ticker", "Ticker", Ticker, &ticker_functions, 1, 1 },
{ "/Utility functions/Track-Hold", "Track-Hold", TrackHold, &trackhold_functions, 2, 1 },
{ "/Utility functions/Edge-Sample", "Edge-Sample", EdgeSample, &edgesample_functions, 2, 2 },
{ "/I-O functions" },
{ "/I-O functions/Get", "Get", Get, &get_functions, 0, 1 },
{ "/I-O functions/Print", "Print", Print, &print_functions, 1, 0 },
{ "/I-O functions/Graph", "Graph", Graph, &graph_functions, 2, 0 },
{ "/I-O functions/Chart", "Chart", Chart, &chart_functions, 2, 0 },
{ "/Other functions" },
{ "/Other functions/Module", "Module", Module, &module_functions, 0, 0 },
{ "/Other functions/Perl", "Perl", Perl, &perl_functions, 0, 0 },
{ "/Other functions/Socket", "Socket", Socket, &socket_functions, 0, 0 },
{ NULL }
};

void register_functions ()
{
int i;
	for (i=0; functions[i].mentry != NULL; i++)
		if (functions[i].fname)
			{
			if (functions[i].funcext && functions[i].funcext->fregstr) functions[i].funcext->fregstr (&(functions[i])); /* run the register function */
			add_menu_item (functions[i].mentry, NULL, create_finst, (int)(&(functions[i])), NULL);
			}
		else add_menu_item (functions[i].mentry, NULL, NULL, 0, "<Branch>");
}
