/*
 *  The graphical definition element of deltaFSD
 *  gutils.c - graphical utilities
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */	
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */

# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <math.h>
# include <sys/timeb.h>
# include "engine.h"
# include "gutils.h"

struct window_data root_window;
	
struct box box_top;	
boxp bpstart = &box_top; /* at any time this points to the first box in the current display chain */

//
// The create utilities
//
boxp create_box (gdouble x, gdouble y, gdouble w, gdouble h)
{
boxp bp;
	bp = calloc (1, sizeof(*bp)); /* we rely on calloc to force NULL into all of the box's pointers */
	bp->x = x;
	bp->y = y;
	bp->w = w;
	bp->h = h;
	return bp;
}

boxp new_box (boxp starter, gdouble x, gdouble y)
{ /* the new box is always added just behind the head of the list (the containing module) */
boxp bp;
	bp = create_box (x, y, BOX_WIDTH, BOX_HEIGHT);
	bp->next = starter->next;
	if (starter->next) starter->next->prev = bp;
	starter->next = bp;
	bp->prev = starter;
	bp->mod_top = starter;
	return bp;
}
sstubp new_srcstub (boxp bp, int numb, varo vp)
{
sstubp sp;
	sp = malloc (sizeof (*sp));
	sp->next = bp->ssbp; /* link it in to the list */
	bp->ssbp = sp;
	sp->numb = numb; /* used to simplify the display */
	sp->vp = vp;
	sp->bp = bp;
	sp->firstl = NULL; /* no lines yet */
	return sp;
} 
dstubp new_dststub (boxp bp, int numb, vari vp)
{
dstubp sp;
	sp = malloc (sizeof (*sp));
	sp->next = bp->dsbp; /* link it in to the list */
	bp->dsbp = sp;
	sp->numb = numb; /* used to simplify the display */
	sp->vp = vp;
	sp->bp = bp;
	sp->lp = NULL; /* no line yet */
	return sp;
} 
void strt_seg (linp lp, gdouble x, gdouble y)
{
	lp->frstseg = malloc (sizeof (*(lp->frstseg)));
	lp->frstseg->next = NULL;
	lp->frstseg->x = x; 
	lp->frstseg->y = y;
	lp->frstseg->lp = lp;
}

lsp new_seg (lsp ls, gdouble x, gdouble y)
/* attach a segment to (ie after) a segment */
{
	ls->next = malloc (sizeof (*(ls->next)));
	ls->next->next = NULL;
	ls->next->x = x;
	ls->next->y = y;
	ls->next->lp = ls->lp;
	return ls->next;
}

lsp new_line (sstubp sp, gdouble x, gdouble y)
{ /* always add the line at the head of the list for that stub */
linp lp;		
	lp = malloc (sizeof (*lp)); /* create the data structure */
	lp->srcstub = sp; /* link the line to the source stub */
	lp->dststub = NULL;
	lp->frstseg = NULL;
	lp->next = sp->firstl;
	sp->firstl = lp; /* link in the line */
	strt_seg (lp, x, y);
	return lp->frstseg;
}

//
// The draw utilities
//
void draw_srcstub_and_line (GtkWidget *widget, boxp bp, sstubp sp)
{ /* the stub here is the source - an output */
linp lp;
	/* draw the stub */	
	gdk_draw_rectangle (root_window.pixmap, root_window.black, FALSE, OSTUBPOSX(sp), OSTUBPOSY(sp), STUBLEN, 1);
	/* we only need to draw unterminated lines - those with a dest stub will be draw from there */
	for (lp = sp->firstl; lp != NULL; lp = lp->next) /* there can be several lines with this stub as source */
		if (lp->dststub == NULL) 
			draw_line (widget, lp, LINSTARTX(sp), LINSTARTY(sp), 0.0, 0.0);
}

void draw_dststub_and_line (GtkWidget *widget, boxp bp, dstubp sp)
{ /* the stub here is the destination - an input */
	/* draw the stub */
	gdk_draw_rectangle (root_window.pixmap, root_window.black, FALSE, ISTUBPOSX(sp), ISTUBPOSY(sp), STUBLEN, 1);
	/* and its line - if one exists - a destination will have 0 or 1 line */
	if (sp->lp != NULL) draw_line (widget, sp->lp, LINSTARTX(sp->lp->srcstub), LINSTARTY(sp->lp->srcstub), LINENDX(sp), LINENDY(sp));
}

void draw_box (GtkWidget *widget, boxp bp)
{
sstubp sp;
dstubp dp;
	/* draw the box */
	gdk_draw_rectangle (root_window.pixmap, root_window.black, FALSE , bp->x, bp->y, bp->w, bp->h);
	/* and its hot spot */
	gdk_draw_rectangle (root_window.pixmap, root_window.black, TRUE , bp->x, bp->y, HOTSPOT_W, HOTSPOT_H);
	/* three lines of text */
	if (bp->text1)
		gdk_draw_text (root_window.pixmap, root_window.font, root_window.black, bp->x+TEXT1_X, bp->y+TEXT1_Y, bp->text1, strlen (bp->text1));
	if (bp->text2)
		gdk_draw_text (root_window.pixmap, root_window.font, root_window.black, bp->x+TEXT2_X, bp->y+TEXT2_Y, bp->text2, strlen (bp->text2));
	if (bp->text3)
		gdk_draw_text (root_window.pixmap, root_window.font, root_window.black, bp->x+TEXT3_X, bp->y+TEXT3_Y, bp->text3, strlen (bp->text3));
	/* now the source and destination stubs and lines */
	for (sp = bp->ssbp; sp != NULL; sp = sp->next) draw_srcstub_and_line (widget, bp, sp);
	for (dp = bp->dsbp; dp != NULL; dp = dp->next) draw_dststub_and_line (widget, bp, dp);
}

void draw_boxes (GtkWidget *widget)
{
boxp bp; 
	for (bp = bpstart; bp != NULL; bp = bp->next) 
		if (bp != &box_top) draw_box (widget, bp); /* no need to draw the top level master box */
}

void update_drawing_area (GtkWidget *widget)
{ 
GdkRectangle update_rect;
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = widget->allocation.width;
	update_rect.height = widget->allocation.height;
	/* apply it */
	gtk_widget_draw (widget, &update_rect);
}

void draw_line (GtkWidget *widget, linp lp, gdouble startx, gdouble starty, gdouble endx, gdouble endy)
{
lsp ls;
	for (ls = lp->frstseg; ls != NULL; ls = ls->next)
		{
		gdk_draw_line (root_window.pixmap, root_window.black, startx, starty, ls->x, starty);
		startx = ls->x;
		if (endx != 0.0  && ls->next == NULL) break;
		gdk_draw_line (root_window.pixmap, root_window.black, startx, starty, startx, ls->y);
		starty = ls->y;
		}
	if (endx != 0.0) /* lines without a destination are drawn to the current x, y */
		{
		gdk_draw_line (root_window.pixmap, root_window.black, startx, starty, startx, endy);
		gdk_draw_line (root_window.pixmap, root_window.black, startx, endy, endx, endy);
		}
}

void print_module (boxp bp)
{
sstubp ssbp;
dstubp dsbp;
int counter;
linp lp;
	if (bp == NULL) return; /* all done */ 
	printf ("box at %d %d function %s\n", (int)bp->x, (int)bp->y, bp->fp->funct->fname);
	for (ssbp = bp->ssbp; ssbp != NULL; ssbp = ssbp->next)
		for (lp = ssbp->firstl; lp != NULL; lp = lp->next) 
			if (lp->dststub) printf ("source stub with line to %s\n", lp->dststub->bp->fp->funct->fname);
			else printf ("source stub with incomplete line\n");
	for (dsbp = bp->dsbp; dsbp != NULL; dsbp = dsbp->next)
		if (dsbp->lp) printf ("destination stub with line to %s\n", dsbp->lp->srcstub->bp->fp->funct->fname);
		else printf ("destination stub with no line\n");
	for (counter = 0; counter < bp->fp->ins; counter++)
		if ((bp->fp->frstin[counter]).ovarb) 
			printf ("input variable with source value %f\n", (bp->fp->frstin[counter]).ovarb->current_value); 
		else printf ("input variable with no source\n");
	for (counter = 0; counter < bp->fp->outs; counter++)
		printf ("output variable with source value %f\n", (bp->fp->frstout[counter]).current_value); 
	if (bp->chld) print_module (bp->chld->next); /* print the chlid branch */
	print_module (bp->next); /* print the next one along this branch */
}

int redraw_callback (windp wp)
{ /* preparatory to threads redrawing is done as a time out callback */
	if (!wp->display_request) return (1); /* nothing has changed - no need to redraw */
	/* we clear the entire pixmap and draw the boxes with their lines and text */
	gdk_draw_rectangle (wp->pixmap, wp->white, TRUE, 0, 0, 
		wp->drawing_area->allocation.width, wp->drawing_area->allocation.height);
	/* and always redraw everything - even if only moving lines */
	/* can't erase and redraw lines without erasing things they are on top of */
	draw_boxes (wp->main_vbox); /* this will also draw all lines */
	update_drawing_area (wp->drawing_area);
	wp->display_request = 0;
	return (1);
}

void redraw_request (char* motivation)
{
#ifdef DUMP
	puts ("-------------------------------------------");
	puts (motivation);
	print_module (box_top.next);
#endif
	root_window.display_request = 1;
}

//
// The find utilities
//
int seg_tst (gdouble x, gdouble y, gdouble sx1, gdouble sy1, gdouble sx2, gdouble sy2)
{ /* test if the give line segment is close enough to the given line */
	if (x<max(sx1, sx2) + LOCATION && x>min(sx1, sx2) - LOCATION 
			&& y<max(sy1, sy2) + LOCATION && y>min(sy1, sy2) - LOCATION) return 1;	
	return 0;
}

dstubp find_dststub (gdouble x, gdouble y)
{ /* check all (destination) stubs to see if any are close the current position */
boxp bp;
dstubp sp;
	for (bp = bpstart; bp != NULL; bp = bp->next)
		for (sp = bp->dsbp; sp != NULL; sp = sp->next)
			if (seg_tst (x, y, ISTUBPOSX(sp), ISTUBPOSY(sp), ISTUBENDX(sp), ISTUBENDY(sp)))
				if (!sp->lp)  return sp; /* only unused input stubs can be destinations */
	return NULL;
}

sstubp find_srcstub (gdouble x, gdouble y)
{ /* check all (source) stubs to see if any are close the current position */
boxp bp;
sstubp sp;
	for (bp = bpstart; bp != NULL; bp = bp->next)
		for (sp = bp->ssbp; sp != NULL; sp = sp->next)
			if (seg_tst (x, y, OSTUBPOSX(sp), OSTUBPOSY(sp), OSTUBENDX(sp), OSTUBENDY(sp)))
				 return sp; /* any output stub can be a source */
	return NULL;
}

lsp check_xseg (linp lp, gdouble x, gdouble y)
{ /* checks for x, y close to a vertical (x) segment of the given line */
lsp ls;
gdouble prevy;
	if (lp == NULL || lp->frstseg == NULL) return NULL; /* no line for this stub */
	prevy = LINSTARTY(lp->srcstub);
	for (ls = lp->frstseg; ls != NULL; ls = ls->next)
		{
		if (ls->next == NULL) break;
		if (seg_tst (x, y, ls->x, prevy, ls->x, ls->y)) return ls;
		prevy = ls->y;
		}
	if (lp->dststub && seg_tst (x, y, ls->x, prevy, ls->x, LINENDY(lp->dststub)))
		return ls; /* the final check is for the segment to the destination stub */
	return NULL;			
}

lsp find_xseg (gdouble x, gdouble y)
{ /* find if x, y are close to a vertical (x) segment of any line */
boxp bp;
dstubp dsp;
sstubp ssp;
linp lp;
lsp ls;
	for (bp = bpstart; bp != NULL; bp = bp->next) /* look in every box */
		{
		for (dsp = bp->dsbp; dsp != NULL; dsp = dsp->next) /* and every destination stub */
			if ((ls = check_xseg (dsp->lp, x, y))) return ls;
		for (ssp = bp->ssbp; ssp != NULL; ssp = ssp->next) /* and every source stub stub */
			for (lp = ssp->firstl; lp != NULL; lp = lp->next) /* and every line with no destination */
				if (!lp->dststub && (ls = check_xseg (lp, x, y))) 
					return ls;
		}
	return NULL;
}

lsp check_yseg (linp lp, gdouble x, gdouble y)
{ /* checks for x, y close to a horizontal (y) segment of the given line */
lsp ls;
	if (lp == NULL || lp->frstseg == NULL) return NULL; /* no line for this stub */
	if (seg_tst (x, y, LINSTARTX(lp->srcstub), LINSTARTY(lp->srcstub), lp->frstseg->x, LINSTARTY(lp->srcstub)))
		return (lp->frstseg); /* check the first line - ie the stub extension */
	for (ls = lp->frstseg; ls->next != NULL; ls = ls->next)
		if (seg_tst (x, y, ls->x, ls->y, ls->next->x, ls->y)) return ls;
	if (lp->dststub && seg_tst (x, y, ls->x, LINENDY(lp->dststub), LINENDX(lp->dststub), LINENDY(lp->dststub)))
		return (ls);  /* check the last line - ie the stub extension */
	return NULL;			
}							
lsp find_yseg (gdouble x, gdouble y)
{ /* find if x, y are close to a horizontal (y) segment of any line */
boxp bp;
dstubp dsp;
sstubp ssp;
linp lp;
lsp ls;
	for (bp = bpstart; bp != NULL; bp = bp->next) /* look in every box */
		{
		for (dsp = bp->dsbp; dsp != NULL; dsp = dsp->next) /* and every destination stub */
			if ((ls = check_yseg (dsp->lp, x, y))) return ls;
		for (ssp = bp->ssbp; ssp != NULL; ssp = ssp->next) /* and every source stub stub */
			for (lp = ssp->firstl; lp != NULL; lp = lp->next) /* and every line with no destination */
				if (!lp->dststub && (ls = check_yseg (lp, x, y))) 
					return ls;
		}
	return NULL;
}

linp find_line (gdouble x, gdouble y)
{
lsp sp;
	if ((sp = find_xseg (x, y))) return sp->lp;
	if ((sp = find_yseg (x, y))) return sp->lp;
	return NULL;
}
boxp find_full_box (gdouble x, gdouble y)
{ /* anywhere in the box */
boxp bp;
	for (bp = bpstart; bp != NULL; bp = bp->next)
		if (x>bp->x && x<(bp->x + bp->w) && y>bp->y && y<(bp->y + bp->h)) return bp;
	return NULL; /* nothing found */
}

boxp find_box_hotspot (gdouble x, gdouble y)
{ /* just the top left corner */
boxp bp;
	for (bp = bpstart; bp != NULL; bp = bp->next)
		if (x>bp->x && x<bp->x+HOTSPOT_W && y>bp->y && y<bp->y+HOTSPOT_H) return bp;
	return NULL; /* nothing found */
}

/*
// The line completion utilities
*/
void variable_linkage (dstubp dp, sstubp sp)
{	/* link an input (of a function) to an output variable */
funp fp;
	if (!dp || !sp) return; /* should test before call ?? fix */
	dp->vp->ovarb = sp->vp;
	/* link the function into those dependent on the variable */
	fp = malloc (sizeof (*fp)); /* create a new list entry */
	fp->depfun = dp->bp->fp;	/* point to the input function */
	fp->depvar = dp->vp;	/* point to this variable */
	fp->next = sp->vp->frstf; /* get the existing top of list */
	sp->vp->frstf = fp; /* reattach the list */
}

sstubp get_original_source (sstubp sp)
{
dstubp dsbp;
	if (sp->bp->chld)
		for (dsbp = sp->bp->chld->dsbp; dsbp != NULL; dsbp = dsbp->next)
			if (sp->numb == dsbp->numb && dsbp->lp)
				return (get_original_source (dsbp->lp->srcstub));
	if (sp->bp->prnt)
		for (dsbp = sp->bp->prnt->dsbp; dsbp != NULL; dsbp = dsbp->next)
			if (sp->numb == dsbp->numb && dsbp->lp)
				return (get_original_source (dsbp->lp->srcstub));
	if (sp->bp->chld || sp->bp->prnt) return NULL; /* module but no line */
	return (sp);
}
	
dstubp get_final_destination (dstubp dp)
{ /* works through an indefinite sequence of module connections to find a "real" function */
sstubp ssbp;
linp lp;
	if (dp->bp->chld) /* find the matching stub for the child */
		for (ssbp = dp->bp->chld->ssbp; ssbp != NULL; ssbp = ssbp->next)
			if (dp->numb == ssbp->numb)
				for (lp = ssbp->firstl; lp != NULL; lp = lp->next)
					return (get_final_destination (lp->dststub));
	if (dp->bp->prnt)
		for (ssbp = dp->bp->prnt->ssbp; ssbp != NULL; ssbp = ssbp->next)
			if (dp->numb == ssbp->numb)
				for (lp = ssbp->firstl; lp != NULL; lp = lp->next)
					return (get_final_destination (lp->dststub));
	if (dp->bp->chld || dp->bp->prnt) return NULL; /* module but no line */
	return (dp);
}

void complete_line (linp lp)
{ /* insert variable - the src and dst stubs will already have been set */
	variable_linkage (get_final_destination (lp->dststub), get_original_source(lp->srcstub));
	lp->dststub->lp = lp; /* join the line to the destination stub */
	process_func (lp->srcstub->bp->fp);		/* recalculate - starting with our new function */
	redraw_request ("Completed line");
}	 
