/*
 *  The graphical definition element of deltaFSD
 *  gutils.h - header file for gutils.c - graphical utilities 
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */    
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */



#ifndef _GUTILS_H_
#define _GUTILS_H_

# include <gtk/gtk.h>
# include "engine.h"
# include "main.h"

# define BOX_STARTX -100.0
# define BOX_STARTY -100.0 /* draw it out of the window */

# define MAIN_WINDOWW 900.0
# define MAIN_WINDOWH 600.0


struct window_data
	{
	GtkWidget *main_vbox;
	GtkWidget *drawing_area; /*fix */
	GdkPixmap *pixmap; /* Backing pixmap for drawing area */
	GdkGC *black;
	GdkGC *white;
	GdkGC *gray;
	GdkFont *font;
	int display_request;
	};
typedef struct window_data *windp;
extern struct window_data root_window;



# define LOCATION 5 /* distance to locate stubs and lines */

/* dimensions are relative to the top left corner of the box */
# define BOX_WIDTH 90.0
# define BOX_HEIGHT 60.0
# define HOTSPOT_W 10.0
# define HOTSPOT_H 10.0
# define TEXT1_X 15.0
# define TEXT1_Y 16.0
# define TEXT2_X 5.0
# define TEXT2_Y 32.0
# define TEXT3_X 5.0
# define TEXT3_Y 48.0
# define TEXTAREA_X 5.0
# define TEXTAREA_Y 16.0
# define TEXTAREA_W (BOX_WIDTH-TEXTAREA_X)
# define TEXTAREA_H (BOX_HEIGHT-TEXTAREA_Y)


# define min(x,y) ((x)>(y)?(y):(x))
# define max(x,y) ((x)<(y)?(y):(x))


typedef struct lseg* lsp;
typedef struct src_stub* sstubp;
typedef struct dst_stub* dstubp;
//typedef struct box* boxp;
//typedef struct line* linp;


extern boxp bpstart;
extern struct box box_top;

typedef enum {IN_STUB, OUT_STUB, MIN_STUB, MOUT_STUB} Stubtype; /* stub types */
struct src_stub
{
	int numb; /* used to simplify the display and associate child/parent stubs in modules */
	sstubp next;
	linp firstl;	/* the lines from this stub  - this is needed for modules where the 1:1 lines:variables does hold */
	boxp bp;
	varo vp;
};
struct dst_stub
{
	int numb; /* used to simplify the display and associate child/parent stubs in modules */
	dstubp next;
	linp lp;		/* the one line to this stub */
	boxp bp;
	vari vp;
};
# define STUBLEN 10.0
# define STUBINSET 5.0 /* in from top/bottom edge of box */
# define STUBYAREA(h) ((h)-(2*STUBINSET)) /* spread n stubs over this length of the side */
# define STUBYOFF(h,n,t) (STUBINSET+((STUBYAREA(h))*(2*(n)+1)/(2*(t))))
# define STOBPOS(bp,snum) STUBYOFF((bp->h),snum,(bp->prnt?(bp->fp->ins):(bp->fp->outs)))
# define STIBPOS(bp,snum) STUBYOFF((bp->h),snum,(bp->prnt?(bp->fp->outs):(bp->fp->ins)))
# define STUBNUM(sp) (sp->numb)
# define BOXRIGHT(sp) ((sp->bp->x)+(sp->bp->w))
# define BOXTOP(sp) (sp->bp->y)
# define BOXLEFT(sp) (sp->bp->x)
# define OSTUBPOSY(sp) (BOXTOP(sp)+STOBPOS(sp->bp,sp->numb))
# define ISTUBPOSY(sp) (BOXTOP(sp)+STIBPOS(sp->bp,sp->numb))
# define OSTUBPOSX(sp) (sp->bp->prnt?BOXLEFT(sp)-STUBLEN:BOXRIGHT(sp))
# define ISTUBPOSX(sp) (sp->bp->prnt?BOXRIGHT(sp):BOXLEFT(sp)-STUBLEN)
# define OSTUBENDX(sp) (sp->bp->prnt?BOXLEFT(sp):BOXRIGHT(sp)+STUBLEN)
# define ISTUBENDX(sp) (sp->bp->prnt?BOXRIGHT(sp)+STUBLEN:BOXLEFT(sp))
# define OSTUBENDY(sp) (OSTUBPOSY(sp))
# define ISTUBENDY(sp) (ISTUBPOSY(sp))
# define LINSTARTX(sp) (sp->bp->prnt?(sp->bp->x):(sp->bp->x+sp->bp->w+STUBLEN))
# define LINSTARTY(sp) (OSTUBPOSY(sp))
# define LINENDY(sp) (ISTUBPOSY(sp))
# define LINENDX(sp) (sp->bp->prnt?(sp->bp->x+sp->bp->w):(sp->bp->x-STUBLEN))

/*
these macros avoid horrors like
sp->lp->srcstub->bp->y+(5.0+(((sp->lp->srcstub->bp->h)-10.0)*(2*(sp->lp->srcstub->numb)+1)/(2*(sp->lp->srcstub->bp->fp->outs))))
*/

struct box
{
	char *text1;   /* first line on box */
	char *text2;   /* second line on box */
	char *text3;   /* third line on box */
	boxp mod_top;
	boxp next;
	boxp prev;
	boxp prnt; /* used for modules */ 
	boxp chld;	/* used for modules */ 
	int bcount; /* used for output - fix */
	gdouble x;
	gdouble y;
	gdouble w;
	gdouble h;
	dstubp dsbp;	// the list of input stubs for this box
	sstubp ssbp;	// the list of output stubs for this box
	funs fp;
	void *idata;
};

struct lseg
{
	lsp next;
	linp lp;
	gdouble x;
	gdouble y;
};

struct line
{
	lsp frstseg;
	linp next; /* next in the chain from a source stub */
	sstubp srcstub;
	dstubp dststub;
};


lsp new_line (sstubp src, gdouble x, gdouble y);
boxp create_box (gdouble x, gdouble y, gdouble w, gdouble h);
boxp new_box (boxp starter, gdouble x, gdouble y);
void update_all (GtkWidget *widget);
void draw_box (GtkWidget *widget, boxp bp);
void draw_boxes (GtkWidget *widget);
boxp find_box_hotspot (gdouble x, gdouble y);
linp find_line (gdouble x, gdouble y);
void draw_line (GtkWidget *widget, linp lp, gdouble startx, gdouble starty, gdouble endx, gdouble endy);
int seg_tst (gdouble x, gdouble y, gdouble sx1, gdouble sy1, gdouble sx2, gdouble sy2);
lsp find_xseg (gdouble x, gdouble y);
lsp find_yseg (gdouble x, gdouble y);
sstubp find_srcstub (gdouble x, gdouble y);
dstubp find_dststub (gdouble x, gdouble y);
void redraw_request (char* motivation);
int redraw_callback (windp wp);
lsp new_seg (lsp ls, gdouble x, gdouble y);
sstubp new_srcstub (boxp bp, int num, varo vp);
dstubp new_dststub (boxp bp, int num, vari vp);
dstubp get_final_destination (dstubp dp);
sstubp get_original_source (sstubp sp);

#endif /* _GUTILS_H_ */


