/*
 *  The graphical definition element of deltaFSD
 *  main.c - overall program start-up and initialization 
 *	Copyright (C) 2003, 2004 DeDf, 45 Belper Road, Derby, DE1 3EP, UK
 */	
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  The deltaFSD concept was developed by DeDf:
 *
 *		deltafsd@dedf.co.uk
 *
 *		www.dedf.co.uk/deltafsd
 *
 *		DeDf
 *		45 Belper Road
 *		Derby
 *		DE1 3EP
 *		UK
 *
 *		+44 1332 381481
 *
 */

# include <stdlib.h>
# include <gtk/gtk.h>
# include <stdio.h>
# include "main.h"
# include "gutils.h"
# include "function.h"

struct current_select cursel;

Operation_type mmode;


/* create a new backing pixmap of the appropriate size */
gint configure_event( GtkWidget *widget, GdkEventConfigure *event )
{ /* the window size may have changed so we might need a different sized pixmap - ?? but how big */
	if (root_window.pixmap) gdk_pixmap_unref(root_window.pixmap);
	root_window.pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	/* get the standard colors and font */
	root_window.black = widget->style->black_gc;
	root_window.white = widget->style->white_gc;
	root_window.gray = widget->style->mid_gc[0];
	root_window.font = widget->style->font;
	redraw_request ("Configure event"); /* draw the existing diagram onto our new pixmap */
	return TRUE;
}

gint expose_event( GtkWidget *widget, GdkEventExpose *event )
{
	gdk_draw_pixmap(widget->window, root_window.white, root_window.pixmap,
		event->area.x, event->area.y, event->area.x, event->area.y,
		event->area.width, event->area.height);
	return FALSE;
}

void select_obj ( GtkWidget *widget, GdkEventButton *event )
{
	if ((cursel.objct = find_srcstub (event->x, event->y))) cursel.typ = SRCSTUB;
	else if ((cursel.objct = find_dststub (event->x, event->y))) cursel.typ = DSTSTUB;
	else if ((cursel.objct = find_box_hotspot (event->x, event->y))) cursel.typ = BOX;
	else if ((cursel.objct = find_xseg (event->x, event->y))) cursel.typ = XSEGMENT;
	else if ((cursel.objct = find_yseg (event->x, event->y))) cursel.typ = YSEGMENT;
	else cursel.typ = NOTHING;
}
void select_event ( GtkWidget *widget, GdkEventButton *event )
{
	select_obj (widget, event);
	if (cursel.typ != NOTHING) mmode = MOVE; 
	if (cursel.typ == SRCSTUB)
	 {
	 cursel.objct = new_line ((sstubp)cursel.objct, event->x, event->y);
	 cursel.typ = SEGMENT;
	 mmode = NEWSEG;
	 }
}
void box_button_press (boxp bp)
{
	/* if the function for the box has a request function call it */
	if (bp->fp->funct->funcext && bp->fp->funct->funcext->frqst) 
		bp->fp->funct->funcext->frqst(bp);
	else /* we have a funcction without a special handler */
		standard_popup (bp);
}
gint button_press_event( GtkWidget *widget, GdkEventButton *event )
{
	if (event->button == 1 && mmode == MOVE) mmode = SELECT; /* cancel move */
	else if (event->button == 1 && mmode == SELECT) select_event (widget, event);
	if (event->button == 3 && mmode == SELECT)
		{ /* what are we near ? */
		if ((cursel.objct = find_box_hotspot (event->x, event->y)))
			box_button_press (cursel.objct);
		if ((cursel.objct = find_line (event->x, event->y)))
			/* all lines get the same options */
			line_popup (cursel.objct, event->x, event->y);
		}
	return TRUE;
}

gint button_release_event( GtkWidget *widget, GdkEventButton *event )
{
	if (root_window.pixmap == NULL) return TRUE;
	if (event->button == 1) switch (mmode) 
		{
		case NEWSEG: 
			if ((((lsp)cursel.objct)->lp->dststub = find_dststub (event->x, event->y)))
				{/* we're at the end of the line - do the stub linkage */
				complete_line (((lsp)cursel.objct)->lp);
				cursel.typ = NOTHING; mmode = SELECT;	/* revert to select mode */
				}
			else cursel.objct = new_seg ((lsp)cursel.objct, event->x, event->y);
			break;
		case MOVE : mmode = SELECT; break;
		default: break;
		}
	return TRUE;
}

gint motion_notify_event( GtkWidget *widget, GdkEventMotion *event )
{
	int x, y;
	GdkModifierType state;
	if (event->is_hint) gdk_window_get_pointer (event->window, &x, &y, &state);
	else { x = event->x; y = event->y; state = event->state; }
	if (root_window.pixmap != NULL)
		{  
		if (/*state & GDK_BUTTON1_MASK &&*/ mmode == MOVE)
		{ 
		if (cursel.typ == BOX) {	((boxp)cursel.objct)->x = x; ((boxp)cursel.objct)->y = y;}
		if (cursel.typ == XSEGMENT) {((lsp)cursel.objct)->x = x;}
		if (cursel.typ == YSEGMENT) {((lsp)cursel.objct)->y = y;}
		redraw_request ("Motion notify in Move mode");
		}
	if (state & GDK_BUTTON1_MASK && mmode == NEWSEG)
		{ 	 
		if (cursel.typ == SEGMENT) {((lsp)cursel.objct)->x = x; ((lsp)cursel.objct)->y = y;}
		if (cursel.typ == SRCSTUB) {((lsp)cursel.objct)->x = x; ((lsp)cursel.objct)->y = y;}
		if (cursel.typ == DSTSTUB) {((lsp)cursel.objct)->x = x; ((lsp)cursel.objct)->y = y;}
		redraw_request ("Motion notify in Newseg mode");
		}
		}
 	return TRUE;
} 

/* function menu item selected - create the function instance and a box for it */
void create_finst (GtkWidget* w, futs funct)
{
funs fp;
	cursel.objct = new_box (bpstart, BOX_STARTX, BOX_STARTY); 
	fp = function_insert(funct, (boxp)cursel.objct);
	if (funct->funcext && funct->funcext->finit) (*funct->funcext->finit)(fp); /* do any per instance initialization */
	cursel.typ = BOX;
	mmode = MOVE;
// don't update here - a function with no inputs doesn't produce (sensible) output ? */
//  and we must let the dialog boxes complete any set up
}

int main( int argc, char *argv[] )
{
GtkWidget *window, *menubar;

	gtk_init (&argc, &argv);

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	gtk_window_set_title ((GtkWindow *)window, "deltafsd");

	root_window.main_vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), root_window.main_vbox);
	gtk_widget_show (root_window.main_vbox);

	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (quit), NULL);

	/* Create the drawing area */
	root_window.drawing_area = gtk_drawing_area_new ();
	gtk_drawing_area_size (GTK_DRAWING_AREA (root_window.drawing_area), MAIN_WINDOWW, MAIN_WINDOWH);

	/* Signals used to handle backing pixmap */
	gtk_signal_connect (GTK_OBJECT (root_window.drawing_area), "expose_event", (GtkSignalFunc) expose_event, NULL);
	gtk_signal_connect (GTK_OBJECT (root_window.drawing_area), "configure_event", (GtkSignalFunc) configure_event, NULL);

	/* Event signals */
	gtk_signal_connect (GTK_OBJECT (root_window.drawing_area), "motion_notify_event", (GtkSignalFunc) motion_notify_event, NULL);
	gtk_signal_connect (GTK_OBJECT (root_window.drawing_area), "button_press_event", (GtkSignalFunc) button_press_event, NULL);
	gtk_signal_connect (GTK_OBJECT (root_window.drawing_area), "button_release_event", (GtkSignalFunc) button_release_event, NULL);

	gtk_widget_set_events (root_window.drawing_area, GDK_EXPOSURE_MASK
		| GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
		| GDK_POINTER_MOTION_HINT_MASK | GDK_POINTER_MOTION_MASK);

	get_main_menu ((GtkWindow *)window, &menubar);
	gtk_box_pack_start (GTK_BOX (root_window.main_vbox), menubar, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (root_window.main_vbox), root_window.drawing_area, TRUE, TRUE, 0);
	// (change to FALSE, TRUE for compatibility with later gtk versions)

	gtk_widget_show (menubar);
	gtk_widget_show (root_window.drawing_area);
	gtk_widget_show (window); // why does this create problems when built statically with later versions of gtk - fix
	/* startup the display ticker callback - as a precursor to threads */
	gtk_timeout_add (DISPLAY_UPDATE_RATE, (GtkFunction)redraw_callback, &root_window);
	/* off we go */
	gtk_main(); 
	
	return 0;
}
