/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"

/* socket includes */
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#define BUFFER_SIZE 200
#define CHANNEL_COUNT 20

# define LOOKUP_RANGE(window,widg) GTK_OBJECT(GTK_RANGE(lookup_widget (window,widg))->adjustment)
# define LOOKUP_BUTTON(window,widg) GTK_OBJECT(GTK_BUTTON(lookup_widget (window,widg)))
# define LOOKUP_DIAL(window,widg) GTK_OBJECT(GTK_DIAL(lookup_widget (window,widg)))



typedef void (*rfunc)(GtkWidget *widget, float val);
typedef struct 
	{
	GtkWidget *widget; 	/* for output widgets we need to know the widget */
	rfunc func;			/* and the handler function which will accept the data */
	} out_def;
typedef struct 
	{
	int cfd;			/* the file descriptor for the comms channel */
	out_def outs[CHANNEL_COUNT];	/* the output devices */
	} piped_data;

typedef struct
	{					/* input widgets are handled on an individual basis */
	int chan;
	int cfd;
	float value;
	} adj_data;

/* local prototypes */
void set_channels (piped_data *pdp, char *strtptr);
void piped_read (piped_data *pdp, gint source, GdkInputCondition condition);
int establish_connection (piped_data *pdp, char *server, int port);
void dial_handlr (piped_data *pdp, int chan, GtkWidget *widget, void * func);
void piped_write (adj_data *adp);
void scale_handlr (GtkObject *object, piped_data *pdp, int chan);
gboolean scale_event (GtkWidget *widget, adj_data *adp);
gboolean button_event (GtkWidget *widget, adj_data *adp);
void button_pressed_handlr (GtkObject *object, piped_data *pdp, int chan, float value);
void button_released_handlr (GtkObject *object, piped_data *pdp, int chan, float value);

void set_channels (piped_data *pdp, char *strtptr)
{
int chan;
double val;
char *endptr;

	for (;;)
		{
		if (!*strtptr) return;
		chan = strtol (strtptr, &endptr, 10);
		if (strtptr == endptr) return;
		val = strtod (endptr+1, &strtptr);
		if (strtptr == endptr+1) return;
		if (pdp->outs[chan].func) pdp->outs[chan].func(pdp->outs[chan].widget, val);
		if (!*strtptr++) return;
		}
}


void piped_read (piped_data *pdp, gint source, GdkInputCondition condition)
{
char buf[BUFFER_SIZE];
int bcount;
	bcount = read (pdp->cfd, buf, BUFFER_SIZE);
	if (bcount <1 ) return;
	buf[bcount] = 0; /* terminate the string */
	set_channels (pdp, buf);
}

int establish_connection (piped_data *pdp, char *server, int port)
{
struct sockaddr_in conn_sa;
struct hostent *host;
int net_host;
char *net_host_ptr = (char *) &net_host;
int i;
	host = gethostbyname (server);
	if (host == 0 || host->h_length != 4) 
		{
		printf ("hostname lookup failed for %s\n", server);
		return (-1);
		}
	for (i = 0; i < 4; i++) *net_host_ptr++ = host->h_addr[i];
	pdp->cfd = socket (AF_INET, SOCK_STREAM, 0);
	conn_sa.sin_addr.s_addr = net_host;
	conn_sa.sin_port = htons ((short) port);
	conn_sa.sin_family = AF_INET;

	if (connect (pdp->cfd, (struct sockaddr *) &conn_sa, sizeof(conn_sa))) return (-1);
	gdk_input_add(pdp->cfd, GDK_INPUT_READ, (GdkInputFunction)piped_read, (gpointer)pdp);

	return (0);
}

void dial_handlr (piped_data *pdp, int chan, GtkWidget *widget, void * func)
{
	pdp->outs[chan].widget = widget;
	pdp->outs[chan].func = func;	
}

void piped_write (adj_data *adp)
{
char buf[BUFFER_SIZE];
int ssze;
	ssze = snprintf (buf, BUFFER_SIZE, "%d,%f\n", adp->chan, adp->value);
	if (write(adp->cfd, buf, strlen(buf)) < ssze) printf ("Write failed for %s\n", buf);
}

gboolean scale_event (GtkWidget *widget, adj_data *adp)
{
	adp->value = GTK_ADJUSTMENT(widget)->value;
	piped_write (adp);
	return FALSE;
}

void scale_handlr (GtkObject *object, piped_data *pdp, int chan)
{
adj_data *adp;
	adp = malloc (sizeof(adj_data));
	adp->chan = chan;
	adp->cfd = pdp->cfd;
	gtk_signal_connect (object, "value_changed", GTK_SIGNAL_FUNC (scale_event), adp);
}

gboolean button_event (GtkWidget *widget, adj_data *adp)
{
	piped_write (adp);
	return FALSE;
}
void button_pressed_handlr (GtkObject *object, piped_data *pdp, int chan, float value)
{
adj_data *adp;
	adp = malloc (sizeof(adj_data));
	adp->chan = chan;
	adp->value = value;
	adp->cfd = pdp->cfd;
	gtk_signal_connect (object, "pressed", GTK_SIGNAL_FUNC (button_event), adp);
}
void button_released_handlr (GtkObject *object, piped_data *pdp, int chan, float value)
{
adj_data *adp;
	adp = malloc (sizeof(adj_data));
	adp->chan = chan;
	adp->value = value;
	adp->cfd = pdp->cfd;
	gtk_signal_connect (object, "released", GTK_SIGNAL_FUNC (button_event), adp);
}

int main (int argc, char *argv[])
{
  GtkWidget *window1;
  GtkWidget *window2;
piped_data pdata;
int i;

	for (i=0;i<CHANNEL_COUNT;i++) pdata.outs[i].widget = pdata.outs[i].func = NULL;

	gnome_init ("car", VERSION, argc, argv);

	if (establish_connection (&pdata, "localhost", argv[1] ? atoi(argv[1]) : 6739))
		{
		printf ("Connection failed\n");
		exit (1); /* no point in continuing */
		}

/* window 1 - input */
	window1 = create_window1 ();
	gtk_widget_show (window1);
	GTK_ADJUSTMENT(LOOKUP_RANGE(window1, "vscale1"))->upper = 100.0;
	button_pressed_handlr (LOOKUP_BUTTON(window1, "button2"), &pdata, 1, +1.0);  
	button_pressed_handlr (LOOKUP_BUTTON(window1, "button3"), &pdata, 1, -1.0);  
	button_released_handlr (LOOKUP_BUTTON(window1, "button2"), &pdata, 1, 0.0);  
	button_released_handlr (LOOKUP_BUTTON(window1, "button3"), &pdata, 1, 0.0);  
	scale_handlr (LOOKUP_RANGE(window1, "vscale1"), &pdata, 0);  

/* window 2 - output */
	window2 = create_window2 ();
	gtk_widget_show (window2);
	dial_handlr (&pdata, 0, lookup_widget (window2, "dial2"), gtk_dial_set_value);
	dial_handlr (&pdata, 1, lookup_widget (window2, "dial3"), gtk_dial_set_value);

	gtk_main ();
	return 0;
}

