package LCD;

use base qw/Tk::Derived Tk::Canvas/;
use Tievar;

Construct Tk::Widget 'LCD';

%SHAPE = (
'E' => [qw/ 0.3348 1.7783 0.4304 1.6913 0.8652 1.6913 0.9435 1.7783 0.8478 1.8652 0.4130 1.8652/],
'F' => [qw/ 0.2609 1.2130 0.3565 1.1261 0.4348 1.2130 0.3913 1.6478 0.2957 1.7348 0.2174 1.6478/],
'G' => [qw/ 0.4043 1.0826 0.5000 0.9957 0.9348 0.9957 1.0130 1.0826 0.9174 1.1696 0.4826 1.1696/],
'A' => [qw/ 0.3304 0.5174 0.4261 0.4304 0.5043 0.5174 0.4609 0.9522 0.3652 1.0391 0.2870 0.9522/],
'B' => [qw/ 0.4739 0.3870 0.5696 0.3000 1.0043 0.3000 1.0826 0.3870 0.9870 0.4739 0.5522 0.4739/],
'C' => [qw/ 1.0261 0.5174 1.1217 0.4304 1.2000 0.5174 1.1565 0.9522 1.0609 1.0391 0.9826 0.9522/],
'D' => [qw/ 0.9565 1.2130 1.0522 1.1261 1.1304 1.2130 1.0870 1.6478 0.9913 1.7348 0.9130 1.6478/],
'dp' => [qw/ 1.100 1.920 1.164 1.894 1.190 1.830 1.164 1.766 1.100 1.740 1.036 1.766 1.010 1.830 1.036 1.894/],
);

%LLCD = (
    '0' => [qw/a b c d e f G/],
    '1' => [qw/A B c d E F G/],
    '2' => [qw/A b c D e f g/],
    '3' => [qw/A b c d e F g/],
    '4' => [qw/a B c d E F g/],
    '5' => [qw/a b C d e F g/],
    '6' => [qw/a b C d e f g/],
    '7' => [qw/A b c d E F G/],
    '8' => [qw/a b c d e f g/],
    '9' => [qw/a b c d e F g/],
    '-' => [qw/A B C D E F g/],
    'E' => [qw/a b C D e f g/],
    'r' => [qw/A B C D E f g/],
    'o' => [qw/A B C d e f g/],
    '-1' => [qw/A B c d E F g/],
    ' ' => [qw/A B C D E F G/],
);


sub Populate {

    my($self, $args) = @_;
    $self->SUPER::Populate($args);

    $self->ConfigSpecs
    	(
        -digits   		=> ['PASSIVE',	'digits',		'Digits',		4],
        -decimals  		=> ['PASSIVE',	'decimals',		'Decimals',		'auto'],
        -digit_size		=> ['PASSIVE',	'digit_size',	'Digit size',	48],
        -height     	=> [$self,		'height',		'Height',		40], # fix
        -onoutline  	=> ['PASSIVE',	'onoutline',	'Onoutline',	'cyan'],
        -onfill     	=> ['PASSIVE',	'onfill',		'Onfill',		'black'],
        -offoutline 	=> ['PASSIVE',	'offoutline',	'Offoutline',	'white'],
        -offfill		=> ['PASSIVE',	'offfill',		'Offfill',		'gray'],
        -variable		=> ['METHOD',	'variable',		'Variable', 	undef ],
        -width			=> ['SELF',		'width',		'Width',	 	0 ],
        -relief			=> ['SELF',		'relief',		'Relief',		'sunken'],
        -borderwidth	=> ['SELF',		'borderwidth',	'Borderwidth',	2],
    	);
}

sub new 
{ # we need to access the set or default values - new() is the first place they are set 
my ($class,$widget,%args) = @_;
my $obj = $class->SUPER::new($widget,%args);

my $width = $obj->cget(-width);
my $digits = $obj->cget(-digits);
my $digit_size = $obj->cget(-digit_size);

    $obj->configure(-height => 2 * $digit_size);
    $obj->configure(-width => ($digits+0.3) * $digit_size);
	return ($obj);
}
sub set 
{   
	$self = shift;               
my $number = shift;
my $digits = $self->cget(-digits);
my $decimals = $self->cget(-decimals);
my $position  = 0;

#	$self->delete('lcd'); # clear the whole display
    if ($decimals < 0) { $decimals = 0 } # sanity check
    if (!($decimals < $digits)) { $decimals = $digits - 1 } # must leave space for leading zero

    for (split /(\d\.|\d|\D)/, get_string ($digits, $decimals, $number))
    	{
 		@z = split '(\.)', $_;
		draw_digit ($position++, @z ) if (@z && $digits--);
 		}
}

sub get_string 
{
my ($digits, $decimals, $number) = @_;
	
	$_ = $decimals;
	CASE:
		{
		/auto/	and do { $fmat = "%". $digits. "f"; $decimals = 0; last CASE };
		$_ == 0 and do { $fmat = "%".$digits.".".$decimals."f"; last CASE };
 		$fmat = "%".($digits+1).".".$decimals."f";
		}
	if (-(10.0**($digits-$decimals-1)) < $number && $number < (10.0**($digits-$decimals)))
		{ $txt = sprintf ($fmat, $number) }
	else { $txt = "Error" }
	if ($txt =~ /\./) { $digits++ }
	sprintf ("%-".$digits.".".$digits."s", $txt);
}
sub draw_digit
{
my ($pos, $c, $dp) = @_;
my ($outl, $fill);

	$pos *=  $self->cget(-digit_size);
	for (@{$LLCD{$c}}) # if a->A segment is off, no translation => symbol is on
		{ polyshape ($symbol, (($symbol = $_) =~ tr/a-z/A-Z/), $pos) }
	polyshape ('dp', $dp, $pos);
}

sub polyshape 
{
my ($symbol, $state, $pos) = @_;
my (@shape, $outl, $fill);
	if ($state)	{ $outl = $self->cget(-onoutline); $fill = $self->cget(-onfill)}		
	else { $outl = $self->cget(-offoutline); $fill = $self->cget(-offfill)}
	for (@{$SHAPE{$symbol}}) { push @shape , $_ * $self->cget(-digit_size)}
	$self->move( $self->createPolygon("@shape", -tags => 'lcd',
		-outline => $outl, -fill => $fill), $pos, 0);
}
sub variable
{
my ($obj, $vref) = @_;	
	tie ($$vref, 'Tievar', $obj, \&set, $vref);
}


1;