package Meter;

use base qw/Tk::Derived Tk::Canvas/;
use Tievar;

Construct Tk::Widget 'Meter';

use constant DIAL_TAG => 'dial_tag';
use constant VALUE_TAG => 'value_tag';

sub Populate
{
    my($self, $args) = @_;
	$self->SUPER::Populate($args);

    $self->ConfigSpecs(
        -cntrsize		=> ['PASSIVE',	'cntrsize',		'Cntrsize',		5],
        -point_length	=> ['PASSIVE',	'point_length',	'Point length',	65],
        -major_tick		=> ['PASSIVE',	'major_tick',	'Major tick',	5],
        -total_ticks	=> ['PASSIVE',	'total_ticks',	'Total ticks',	25],
        -dial_xpos		=> ['PASSIVE',	'dial_xpos',	'Dial Xpos',	100],
        -dial_ypos		=> ['PASSIVE',	'dial_ypos',	'Dial Ypos',	100],
        -label			=> ['PASSIVE',	'label',		'Label',		'A meter'],
        -label_anchor	=> ['PASSIVE',	'label_anchor',	'Label Anchor',	'se'],
        -font 			=> ['PASSIVE',	'font',			'Font',			'Courier 8'],
        -variable		=> ['METHOD',	'variable',		'Variable', 	undef ],
        -width 			=> ['SELF',		'width',		'Width',		200],
        -height			=> ['SELF',		'height',		'Height',		130],
        -relief			=> ['SELF',		'relief',		'Relief',		'sunken'],
        -borderwidth	=> ['SELF',		'borderwidth',	'Borderwidth',	2],
    );    
}

sub new 
{ # we need to access the set or default values - new() is the first place they are set 
my ($class,$widget,%args) = @_;
my $obj = $class->SUPER::new($widget,%args);

my $cntrsize = $obj->cget(-cntrsize);
my $point_length = $obj->cget(-point_length);
my $major_tick = $obj->cget(-major_tick);
my $total_ticks = $obj->cget(-total_ticks);
my $dial_xpos = $obj->cget(-dial_xpos);
my $dial_ypos = $obj->cget(-dial_ypos);
my $label = $obj->cget(-label);
my $label_anchor = $obj->cget(-label_anchor);
my $font = $obj->cget(-font);
my $width = $obj->cget(-width);
my $height = $obj->cget(-height);

	for (my ($i)=0; $i<=$total_ticks; $i++)
		{ $obj->createLine(radiate ($dial_xpos, $dial_ypos, $point_length+10, 
			($i % $major_tick)?8:15, $i/$total_ticks), -width=>1); }
 	$obj->createArc($dial_xpos-$cntrsize, $dial_ypos-$cntrsize, 
 			$dial_xpos+$cntrsize,$dial_ypos+$cntrsize, 
 		-start=>-1, -extent=>359, -style=>'arc', -width=>2);
	$obj->createLine(radiate($dial_xpos, $dial_ypos, $cntrsize, $point_length, 0),
			-tag=>DIAL_TAG,  -arrow=>'last', -width=>3);
    $obj->createText($width-5, $height-2, -text=>$label, -anchor=>$label_anchor, -font=>$font);
    $obj->createText(5, $height-2, -tags=>VALUE_TAG, -anchor=>'sw', -font=>$font);

	return ($obj);
}

sub set
{
my ($self, $pos) = @_;
my $cntrsize = $self->cget(-cntrsize);
my $dial_xpos = $self->cget(-dial_xpos);
my $dial_ypos = $self->cget(-dial_ypos);
my $point_length = $self->cget(-point_length);

	$self->coords(DIAL_TAG, radiate ($dial_xpos, $dial_ypos, 
		$cntrsize, $point_length, $pos));
	$self->itemconfigure(VALUE_TAG, -text=>$pos);
}

sub radiate 
{
my @coords;
use constant PI => 3.14159265;

	my ($xpos, $ypos, $start, $len, $angle) = @_;
	$coords[0] = $xpos-$start*cos($angle*PI);
	$coords[1] = $ypos-$start*sin($angle*PI);
	$coords[2] = $xpos-($start+$len)*cos($angle*PI);
	$coords[3] = $ypos-($start+$len)*sin($angle*PI);
	return (@coords);
}

sub variable
{
my ($obj, $vref) = @_;	
	tie ($$vref, 'Tievar', $obj, \&set, $vref);
}

1;